#
# Modes (first argument given to scipion).
#

MODE_MANAGER = 'manager'
MODE_PROJECT = 'project'
MODE_LAST = 'last'  # shortcut to 'project last'
MODE_HERE = 'here'  # shortcut to 'project here'
MODE_COLLECTSTATIC = 'collectstatic'
MODE_TESTS = 'tests'  # keep tests for compatibility
MODE_TEST = 'test'  # also allow 'test', in singular
MODE_TEST_DATA = 'testdata'
MODE_HELP = 'help'
MODE_VIEWER = ['viewer', 'view', 'show']

# Installation modes
MODE_PLUGINS = 'plugins'
MODE_INSTALL_PLUGIN = 'installp'
MODE_UNINSTALL_PLUGIN = 'uninstallp'
MODE_INSTALL_BINS = 'installb'
MODE_UNINSTALL_BINS = 'uninstallb'
MODE_CONFIG = 'config'
MODE_APACHE_VARS = 'apache_vars'
MODE_VERSION = 'version'
MODE_RUNPROTOCOL = 'runprotocol'
MODE_PROTOCOLS = 'protocols'
MODE_STATS = 'stats'
MODE_ENV = 'printenv'
MODE_RUN = 'run'
MODE_PYTHON = 'python'
MODE_TUTORIAL = 'tutorial'
MODE_DEMO = ['demo', 'template']
MODE_INSPECT = "inspect"
MODE_UPDATE = 'update'
PLUGIN_MODES = [MODE_UNINSTALL_PLUGIN, MODE_INSTALL_PLUGIN,
                MODE_INSTALL_BINS, MODE_UNINSTALL_BINS]


DEVEL = 'devel'

# Entry points
SCIPION_EP = "scipion"

# python file name: at install or scripts
PLUGIN_MANAGER_PY = 'plugin_manager.py'
PYTHON = 'python'
KICKOFF = 'kickoff.py'


import tkinter as Tkinter
import tkinter.font as tkFont
from tkinter import ttk


class Font_wm(Tkinter.Toplevel):
    def __init__(self, Font=None):

        Tkinter.Toplevel.__init__(self)
        self.mainfont = Font
        self.title('Font ...')

        # Variable
        self.var = Tkinter.StringVar()  # For Font Face
        self.var.set(self.mainfont.actual('family'))
        self.var1 = Tkinter.IntVar()  # for Font Size
        self.var1.set(self.mainfont.actual('size'))
        self.var2 = Tkinter.StringVar()  # For Bold
        self.var2.set(self.mainfont.actual('weight'))
        self.var3 = Tkinter.StringVar()  # For Italic
        self.var3.set(self.mainfont.actual('slant'))
        self.var4 = Tkinter.IntVar()  # For Underline
        self.var4.set(self.mainfont.actual('underline'))
        self.var5 = Tkinter.IntVar()  # For Overstrike
        self.var5.set(self.mainfont.actual('overstrike'))

        # Font Sample
        self.font_1 = tkFont.Font()
        for i in ['family', 'weight', 'slant', 'overstrike', 'underline',
                  'size']:
            self.font_1[i] = self.mainfont.actual(i)

        # Function
        def checkface(event):
            try:
                self.var.set(str(self.listbox.get(self.listbox.curselection())))
                self.font_1.config(family=self.var.get(), size=self.var1.get(),
                                   weight=self.var2.get(),
                                   slant=self.var3.get(),
                                   underline=self.var4.get(),
                                   overstrike=self.var5.get())
            except:
                pass

        def checksize(event):
            try:
                self.var1.set(int(self.size.get(self.size.curselection())))
                self.font_1.config(family=self.var.get(), size=self.var1.get(),
                                   weight=self.var2.get(),
                                   slant=self.var3.get(),
                                   underline=self.var4.get(),
                                   overstrike=self.var5.get())
            except:
                pass

        def applied():
            self.result = (
                self.var.get(), self.var1.get(), self.var2.get(), self.var3.get(),
                self.var4.get(), self.var5.get())
            self.mainfont['family'] = self.var.get()
            self.mainfont['size'] = self.var1.get()
            self.mainfont['weight'] = self.var2.get()
            self.mainfont['slant'] = self.var3.get()
            self.mainfont['underline'] = self.var4.get()
            self.mainfont['overstrike'] = self.var5.get()

        def out():
            self.result = (
                self.var.get(), self.var1.get(), self.var2.get(), self.var3.get(),
                self.var4.get(), self.var5.get())
            self.mainfont['family'] = self.var.get()
            self.mainfont['size'] = self.var1.get()
            self.mainfont['weight'] = self.var2.get()
            self.mainfont['slant'] = self.var3.get()
            self.mainfont['underline'] = self.var4.get()
            self.mainfont['overstrike'] = self.var5.get()
            self.destroy()

        def end():
            self.result = None
            self.destroy()

        # Main window Frame
        self.mainwindow = ttk.Frame(self)
        self.mainwindow.pack(padx=10, pady=10)
        # Main LabelFrame
        self.mainframe = ttk.Frame(self.mainwindow)
        self.mainframe.pack(side='top', ipady=30, ipadx=30, expand='no',
                            fill='both')
        self.mainframe0 = ttk.Frame(self.mainwindow)
        self.mainframe0.pack(side='top', expand='yes', fill='x', padx=10,
                             pady=10)
        self.mainframe1 = ttk.Frame(self.mainwindow)
        self.mainframe1.pack(side='top', expand='no', fill='both')
        self.mainframe2 = ttk.Frame(self.mainwindow)
        self.mainframe2.pack(side='top', expand='yes', fill='x', padx=10,
                             pady=10)
        # Frame in [  main frame]
        self.frame = ttk.LabelFrame(self.mainframe, text='Select Font Face')
        self.frame.pack(side='left', padx=10, pady=10, ipadx=20, ipady=20,
                        expand='yes', fill='both')
        self.frame1 = ttk.LabelFrame(self.mainframe, text='Select Font size')
        self.frame1.pack(side='left', padx=10, pady=10, ipadx=20, ipady=20,
                         expand='yes', fill='both')
        ttk.Entry(self.frame, textvariable=self.var).pack(side='top', padx=5,
                                                          pady=5, expand='yes',
                                                          fill='x')
        self.listbox = Tkinter.Listbox(self.frame, bg='gray70')
        self.listbox.pack(side='top', padx=5, pady=5, expand='yes', fill='both')
        for i in tkFont.families():
            self.listbox.insert(Tkinter.END, i)

        # Frame in [ 0. mainframe]
        self.bold = ttk.Checkbutton(self.mainframe0, text='Bold',
                                    onvalue='bold', offvalue='normal',
                                    variable=self.var2)
        self.bold.pack(side='left', expand='yes', fill='x')
        self.italic = ttk.Checkbutton(self.mainframe0, text='Italic',
                                      onvalue='italic', offvalue='roman',
                                      variable=self.var3)
        self.italic.pack(side='left', expand='yes', fill='x')
        self.underline = ttk.Checkbutton(self.mainframe0, text='Underline',
                                         onvalue=1, offvalue=0,
                                         variable=self.var4)
        self.underline.pack(side='left', expand='yes', fill='x')
        self.overstrike = ttk.Checkbutton(self.mainframe0, text='Overstrike',
                                          onvalue=1, offvalue=0,
                                          variable=self.var5)
        self.overstrike.pack(side='left', expand='yes', fill='x')

        # Frame in [ 1. main frame]
        ttk.Entry(self.frame1, textvariable=self.var1).pack(side='top', padx=5,
                                                            pady=5,
                                                            expand='yes',
                                                            fill='x')
        self.size = Tkinter.Listbox(self.frame1, bg='gray70')
        self.size.pack(side='top', padx=5, pady=5, expand='yes', fill='both')
        for i in range(30):
            self.size.insert(Tkinter.END, i)

        Tkinter.Label(self.mainframe1, bg='white', text='''
ABCDEabcde12345
''', font=self.font_1).pack(expand='no', padx=10, pady=10)

        # Frame in [ 2. mainframe]
        ttk.Button(self.mainframe2, text='   OK   ', command=out).pack(
            side='left', expand='yes', fill='x', padx=5, pady=5)
        ttk.Button(self.mainframe2, text=' Cancel ', command=end).pack(
            side='left', expand='yes', fill='x', padx=5, pady=5)
        ttk.Button(self.mainframe2, text=' Apply  ', command=applied).pack(
            side='left', expand='yes', fill='x', padx=5, pady=5)

        self.listbox.bind('<<ListboxSelect>>', checkface)
        self.size.bind('<<ListboxSelect>>', checksize)


if __name__ == "__main__":
    root = Tkinter.Tk()
    font1 = tkFont.Font()
    Tkinter.Text(root, font=font1).pack()
    Font_wm(Font=font1)
    root.mainloop()



# -*- coding: utf-8 -*-
#!/usr/bin/env python
# **************************************************************************
# *
# * Authors:     Pablo Conesa (pconesa@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
"""
Creates a scipion workflow file (json formatted) base on a template.
The template may have some ~placeholders~ that will be overwritten with values
Template may look like this, separator is "~" and within it you can define:
~title|value|type~
Template string sits at the end of the file ready for a running streaming demo.
"""
import subprocess
import sys
import os
import re
import tkinter as tk
import tkinter.font as tkFont
import traceback

import pyworkflow as pw
import pyworkflow.utils as pwutils
from pyworkflow.gui import Message, dialog
from pyworkflow.plugin import SCIPION_JSON_TEMPLATES, Template
from pyworkflow.project import ProjectSettings
import pyworkflow.gui as pwgui
from pyworkflow.gui.project.base import ProjectBaseWindow
from pyworkflow.gui.widgets import HotButton, Button
from pyworkflow.template import TemplateList
from scipion.constants import SCIPION_EP

# Custom labels
from scipion.utils import getExternalJsonTemplates

FLAG_PARAM = "--"
NOGUI_FLAG = FLAG_PARAM +"nogui"


START_BUTTON = "Start"
LEN_LABEL_IN_CHARS = 30
LABEL_ALIGN_PATTERN = '{:>%s}' % LEN_LABEL_IN_CHARS
LEN_BUTTON_IN_CHARS = 12
BUTTON_ALIGN_PATTERN = "{:^%s}" % LEN_BUTTON_IN_CHARS

FIELD_SEP = '~'
VIEW_WIZARD = 'wizardview'

# Session id
PROJECT_NAME = 'Project name'

# Project regex to validate the session id name
PROJECT_PATTERN = "^\w{2}\d{4,6}-\d+$"
PROJECT_REGEX = re.compile(PROJECT_PATTERN)


class KickoffWindow(ProjectBaseWindow):
    """ Windows to manage all projects. """

    def __init__(self, **kwargs):
        try:
            title = '%s (%s on %s)' % ('Workflow template customizer',
                                       pwutils.getLocalUserName(),
                                       pwutils.getLocalHostName())
        except Exception:
            title = Message.LABEL_PROJECTS

        settings = ProjectSettings()
        self.generalCfg = settings.getConfig()

        ProjectBaseWindow.__init__(self, title, minsize=(800, 350), **kwargs)
        self.viewFuncs = {VIEW_WIZARD: KickoffView}
        self.template = kwargs.get('template', None)
        self.action = Message.LABEL_BUTTON_CANCEL
        self.switchView(VIEW_WIZARD, **kwargs)

    def close(self, e=None):
        self.root.destroy()
        sys.exit(0)

    def getTemplate(self):
        return self.template

    def getAction(self):
        return self.action

    def switchView(self, newView, **kwargs):
        # Destroy the previous view if exists:
        if self.viewWidget:
            self.viewWidget.grid_forget()
            self.viewWidget.destroy()
        # Create the new view: Instantiates KickoffView HERE!.
        self.viewWidget = self.viewFuncs[newView](self.footer, self, template=self.template)
        # Grid in the second row (1)
        self.viewWidget.grid(row=0, column=0, columnspan=10, sticky='news')
        self.footer.rowconfigure(0, weight=1)
        self.footer.columnconfigure(0, weight=1)
        self.view = newView


class KickoffView(tk.Frame):
    def __init__(self, parent, windows, template=None, **kwargs):
        tk.Frame.__init__(self, parent, bg='white', **kwargs)
        self.windows = windows
        self.root = windows.root
        self.vars = {}
        self.checkvars = []
        self.template = template

        bigSize = pwgui.cfgFontSize + 2
        smallSize = pwgui.cfgFontSize - 2
        fontName = pwgui.cfgFontName

        self.bigFont = tkFont.Font(size=bigSize, family=fontName)
        self.bigFontBold = tkFont.Font(size=bigSize, family=fontName,
                                       weight='bold')

        self.projDateFont = tkFont.Font(size=smallSize, family=fontName)
        self.projDelFont = tkFont.Font(size=smallSize, family=fontName,
                                       weight='bold')
        # Body section
        bodyFrame = tk.Frame(self, bg='white')
        bodyFrame.columnconfigure(0, minsize=120)
        bodyFrame.columnconfigure(1, minsize=120, weight=1)
        bodyFrame.grid(row=0, column=0, sticky='news')
        self._fillContent(bodyFrame)

        # Add the create project button
        btnFrame = tk.Frame(self, bg='white')

        btn = HotButton(btnFrame, text=START_BUTTON,
                        font=self.bigFontBold,
                        command=self._onReadDataFromTemplateForm)
        btn.grid(row=0, column=1, sticky='ne', padx=10, pady=10)

        # Add the Import project button
        btn = Button(btnFrame, Message.LABEL_BUTTON_CANCEL,
                     font=self.bigFontBold,
                     command=self._closeCallback)
        btn.grid(row=0, column=0, sticky='ne', pady=10)

        btnFrame.columnconfigure(0, weight=1)
        btnFrame.grid(row=1, column=0, sticky='news')

        self.columnconfigure(0, weight=1)

    def _closeCallback(self):
        self.root.destroy()
        sys.exit()

    def _fillContent(self, frame):
        # Add project name
        self.template.genProjectName()
        self._addPair(PROJECT_NAME, PROJECT_NAME, 1, frame, value=self.template.projectName, pady=(10, 30))
        # Add template params
        self._addTemplateFieldsToForm(frame)

    def _addPair(self, text, title, r, lf, widget='entry', traceCallback=None,  mouseBind=False, value=None, pady=2):
        label = tk.Label(lf, text=text, bg='white', font=self.bigFont)
        label.grid(row=r, column=0, padx=(10, 5), pady=pady, sticky='nes')

        if not widget:
            return

        var = tk.StringVar()

        if value is not None:
            var.set(value)

        if widget == 'entry':
            widget = tk.Entry(lf, width=30, font=self.bigFont,
                              textvariable=var)
            if traceCallback:
                if mouseBind:  # call callback on click
                    widget.bind("<Button-1>", traceCallback, "eee")
                else:  # call callback on type
                    var.trace('w', traceCallback)
        elif widget == 'label':
            widget = tk.Label(lf, font=self.bigFont, textvariable=var)

        self.vars[title] = var
        widget.grid(row=r, column=1, sticky='news', padx=(5, 10), pady=pady)

    def _addTemplateFieldsToForm(self, labelFrame):
        row = 3
        for field in self.template.params.values():
            alias = field.getAlias()
            text = field.getTitle() if alias is None else "%s (%s)" % (field.getTitle(),alias)

            self._addPair(text, field.getTitle(), row, labelFrame, value=field.getValue())
            row += 1

    def _getVar(self, varKey):
        return self.vars[varKey]

    def _getValue(self, varKey):
        return self.vars[varKey].get()

    def _setValue(self, varKey, value):
        return self.vars[varKey].set(value)

    # noinspection PyUnusedLocal
    def _onReadDataFromTemplateForm(self, e=None):
        errors = []

        # Check the entered data
        for field in self.template.params.values():
            newValue = self._getValue(field.getTitle())
            field.setValue(newValue)
            if not field.validate():
                errors.append("%s value does not validate. Value: %s, Type: %s"
                              % (field.getTitle(), field.getValue(),
                                 field.getType()))

        # Do more checks only if there are not previous errors
        if errors:
            errors.insert(0, "*Errors*:")
            self.windows.showError("\n  - ".join(errors))
        else:
            self.template.projectName = self._getValue(PROJECT_NAME)

            # Set parent with the data
            self.windows.template = self.template
            self.windows.action = START_BUTTON
            self.windows.root.quit()
            self.windows.root.withdraw()
            return


def getTemplates():
    """ Get a template or templates either from arguments
        or from the templates directory.
        If more than one template is found or passed, a dialog is raised
        to choose one.
    """
    templateFolder = getExternalJsonTemplates()
    customTemplates = len(sys.argv) > 1
    tempList = TemplateList()
    tempId = None
    if customTemplates:
        fileTemplate = sys.argv[1]
        if os.path.isfile(fileTemplate) and os.path.exists(fileTemplate):
            t = Template("custom_template", fileTemplate)
            tempList.addTemplate(t)
        else:
            tempId = sys.argv[1]
    # Try to find all templates from the template folder and the plugins
    if len(tempList.templates) == 0:
        tempList.addScipionTemplates(tempId)
        if not (tempId is not None and len(tempList.templates) == 1):
            tempList.addPluginTemplates(tempId)

    if not len(tempList.templates):
        raise Exception("No valid file found (*.json.template).\n"
                        "Please, add (at least one) at %s "
                        "or pass it/them as argument(s).\n"
                        "\n -> Usage: scipion template [PATH.json.template]\n"
                        "\n see 'scipion help'\n" % templateFolder)

    return tempList.sortListByPluginName().templates


def chooseTemplate(templates):
    if len(templates) == 1:
        chosenTemplate = templates[0]
    else:
        provider = pwgui.tree.ListTreeProviderTemplate(templates)
        dlg = dialog.ListDialog(None, "Workflow templates", provider,
                                "Select one of the templates.",
                                selectOnDoubleClick=True)

        if dlg.result == dialog.RESULT_CANCEL:
            sys.exit()
        chosenTemplate = dlg.values[0]

    print("Template to use: %s" % chosenTemplate)
    # Replace environment variables
    chosenTemplate.replaceEnvVariables()

    return chosenTemplate


def resolveTemplate(template):
    """ Resolve a template assigning CML params to the template.
    if not enough, a window will pop pup to ask for missing ones only"""
    if not assignAllParams(template):
        wizWindow = KickoffWindow(template=template)
        wizWindow.show()
        return wizWindow.action == START_BUTTON
    else:
        # All parameters have been assigned and template is fully populated
        return True


def assignAllParams(template):
    """
    Assign CML params to the template, if missing params after assignment
    return False
    """
    paramsSetted = 0
    template.parseContent()
    if len(sys.argv) > 2:
        attrList = sys.argv[2:]

        for attr in attrList:
            # skipp --params
            if attr.startswith(FLAG_PARAM):
                continue

            aliasAttr, valAttr = attr.split('=')
            try:
                paramsSetted += template.setParamValue(aliasAttr, valAttr)
            except Exception as e:
                print(pwutils.redStr(e))
                sys.exit(os.EX_DATAERR)

        return len(template.params) == paramsSetted
    return False


def launchTemplate(template):
    """ Launches a resolved template"""
    try:
        workflow = template.createTemplateFile()
    except Exception as e:
        workflow = None
        errorStr = "Couldn't create the template.\n" + str(e)
        print(errorStr)
        traceback.print_exc()

    if workflow is not None:
        # Create the project
        if not template.projectName:
            template.genProjectName()
        createProjectFromWorkflow(workflow, template.projectName)


def createProjectFromWorkflow(workflow, projectName):

    scipion = SCIPION_EP
    scriptsPath = pw.join('project', 'scripts')

    # Create the project
    createProjectScript = os.path.join(scriptsPath, 'create.py')
    os.system("python -m %s  python %s %s %s" % (scipion, createProjectScript, projectName, workflow))

    # Schedule the project
    scheduleProjectScript = os.path.join(scriptsPath, 'schedule.py')
    print("Scheduling project %s" % projectName)
    subprocess.Popen(["python", "-m",scipion, "python" , scheduleProjectScript, projectName])

    if launchGUI():
        # Launch scipion
        subprocess.Popen(["python", "-m", scipion, projectName])

def launchGUI():
    """Checks if project GUI has to be launched. Only if --noGUI param is found in sys.argv it will return False"""
    for arg in sys.argv:
        if (NOGUI_FLAG) == arg.lower():
            return False

    # Not found, launch GUI
    return True

def main():
    templates = getTemplates()
    chosenTemplate = chooseTemplate(templates)
    if resolveTemplate(chosenTemplate):
        launchTemplate(chosenTemplate)
    else:
        sys.exit(3)

if __name__ == "__main__":
    main()



import requests
import os
import re
import sys
import json
import pkg_resources
from pkg_resources import parse_version

from .funcs import Environment, VOID_TGZ
from pwem import Domain
from pyworkflow.utils import redStr, yellowStr
from pyworkflow.utils.path import cleanPath
from pyworkflow import LAST_VERSION, CORE_VERSION, OLD_VERSIONS, Config
from importlib import reload

NULL_VERSION = "0.0.0"
# This constant is used in order to install all plugins taking into account a
# json file
DEVEL_VERSION = "999.9.9"

REPOSITORY_URL = Config.SCIPION_PLUGIN_JSON

if REPOSITORY_URL is None:
    REPOSITORY_URL = Config.SCIPION_PLUGIN_REPO_URL

PIP_BASE_URL = 'https://pypi.python.org/pypi'
PIP_CMD = '{0} -m pip install %(installSrc)s'.format(
    Environment.getPython())

PIP_UNINSTALL_CMD = '{0} -m pip uninstall -y %s'.format(
    Environment.getPython())

versions = list(OLD_VERSIONS) + [LAST_VERSION]


class PluginInfo(object):

    def __init__(self, pipName="", name="", pluginSourceUrl="", remote=True,
                 plugin=None, **kwargs):
        self.pipName = pipName
        self.name = name
        self.pluginSourceUrl = pluginSourceUrl
        self.remote = remote

        # things from pypi
        self.homePage = ""
        self.summary = ""
        self.author = ""
        self.email = ""
        self.compatibleReleases = {}
        self.latestRelease = ""

        # things we have when installed
        self.dirName = ""
        self.pipVersion = ""
        self.binVersions = []
        self.pluginEnv = None

        # Distribution
        self._dist = None
        self._plugin = plugin
        if self.remote:
            self.setRemotePluginInfo()
        else:
            self.setFakedRemotePluginInfo()

        self.setLocalPluginInfo()  # get local info if installed

    # ###################### Install funcs ############################

    def install(self):
        """Installs both pip module and default binaries of
        the plugin"""
        self.installPipModule()
        self.installBin()
        self.setLocalPluginInfo()

    def _getDistribution(self):
        if self._dist is None:
            try:
                self._dist = pkg_resources.get_distribution(self.pipName)
            except:
                pass
        return self._dist

    def _getPlugin(self):
        if self._plugin is None:

            try:
                dirname = self.getDirName()
                self._plugin = Config.getDomain().getPlugin(dirname)
            except:
                pass
        return self._plugin

    def hasPipPackage(self):
        """Checks if the current plugin is installed via pip"""
        return self._getDistribution() is not None

    def isInstalled(self):
        """Checks if the current plugin is installed (i.e. has pip package).
        NOTE: we might wanna change definition of isInstalled, hence the extra function."""
        reload(pkg_resources)
        return self.hasPipPackage()

    def installPipModule(self, version=""):
        """Installs the version specified of the pip plugin, as long as it is compatible
        with the current Scipion version. If no version specified, will install latest
        compatible one."""
        environment = Environment()

        if not version:
            version = self.latestRelease
        elif version not in self.compatibleReleases:
            if self.compatibleReleases:
                print('%s version %s not compatible with current Scipion '
                      'version %s.' % (self.pipName, version, LAST_VERSION))
                print("Please choose a compatible release: %s" % " ".join(
                    self.compatibleReleases.keys()))
            else:
                print("%s has no compatible versions with current Scipion "
                      "version %s." % (self.pipName, LAST_VERSION))
            return False

        if version == NULL_VERSION:
            print("Plugin %s is not available for this Scipion %s yet" % (self.pipName, LAST_VERSION))
            return False

        if self.pluginSourceUrl:
            if os.path.exists(self.pluginSourceUrl):
                # install from dir in editable mode
                installSrc = '-e %s' % self.pluginSourceUrl
                target = "%s*" % self.pipName
            else:
                # path doesnt exist, we assume is git and force install
                installSrc = '--upgrade git+%s' % self.pluginSourceUrl
                target = "%s*" % self.pipName.replace('-', '_')
        else:
            # install from pypi
            installSrc = "%s==%s" % (self.pipName, version)
            target = "%s*" % self.pipName.replace('-', '_')

        cmd = PIP_CMD % {'installSrc': installSrc}

        pipModule = environment.addPipModule(self.pipName,
                                             target=target,
                                             pipCmd=cmd)

        # check if we're doing a version change of an already installed plugin
        reloadPkgRes = self.isInstalled()

        environment.execute()
        # we already have a dir for the plugin:
        if reloadPkgRes:
            # if plugin was already installed, pkg_resources has the old one
            # so it needs a reload
            reload(pkg_resources)
            self.dirName = self.getDirName()
            Domain.refreshPlugin(self.dirName)
        return True

    def installBin(self, args=None):
        """Install binaries of the plugin. Args is the list of args to be
           passed to the install environment."""
        environment = self.getInstallenv(envArgs=args)
        environment.execute()

    def uninstallBins(self, binList=None):
        """Uninstall binaries of the plugin.
        - binList: if  given, will uninstall the binaries in it. The binList
                   may contain strings with only the name of the binary or
                   name and version in the format name-version"""
        if binList is None:
            binList = self.binVersions

        binFolder = Environment.getEmFolder()
        for binVersion in binList:
            f = os.path.join(binFolder, binVersion)
            if os.path.exists(f):
                print('Removing %s binaries...' % binVersion)
                realPath = os.path.realpath(f)  # in case its a link
                cleanPath(f, realPath)
                print('Binary %s has been uninstalled successfully ' % binVersion)
            else:
                print('The binary %s does not exist ' % binVersion)
        return

    def uninstallPip(self):
        """Removes pip package from site-packages"""
        print('Removing %s plugin...' % self.pipName)
        import subprocess
        args = (PIP_UNINSTALL_CMD % self.pipName).split()
        subprocess.call(PIP_UNINSTALL_CMD % self.pipName, shell=True,
                        stdout=sys.stdout,
                        stderr=sys.stderr)

    # ###################### Remote data funcs ############################

    def getPipJsonData(self):
        """"Request json data from pypi, return json content"""
        pipData = requests.get("%s/%s/json" % (PIP_BASE_URL, self.pipName))
        if pipData.ok:
            pipData = pipData.json()
            return pipData
        else:
            print("Warning: Couldn't get remote plugin data for %s" % self.pipName)
            return {}

    def getCompatiblePipReleases(self, pipJsonData=None):
        """Get pip releases of this plugin that are compatible with
         current Scipion version. Returns dict with all compatible releases and
         a special key "latest" with the most recent one."""

        if pipJsonData is None:
            pipJsonData = self.getPipJsonData()

        reg = r'scipion-([\d.]*\d)'
        releases = {}
        latestCompRelease = NULL_VERSION

        for release, releaseData in pipJsonData['releases'].items():
            releaseData = releaseData[0]
            scipionVersions = [parse_version(v)
                               for v in re.findall(reg,
                                                   releaseData['comment_text'])]
            if len(scipionVersions) != 0:
                releases[release] = releaseData
                if any([v == parse_version(CORE_VERSION)
                          for v in scipionVersions]):
                    if parse_version(latestCompRelease) < parse_version(release):
                        latestCompRelease = release
            else:
                print(yellowStr("WARNING: %s's release %s did not specify a "
                             "compatible Scipion version. Please, remove this "
                             "release from pypi") % (self.pipName, release))

        releases['latest'] = latestCompRelease
        return releases

    def setRemotePluginInfo(self):
        """Sets value for the attributes that need to be obtained from pypi"""
        reg = r'scipion-([\d.]*\d)'
        pipData = self.getPipJsonData()
        if not pipData:
            return
        info = pipData['info']
        releases = self.getCompatiblePipReleases(pipJsonData=pipData)

        self.homePage = info['home_page']
        self.summary = info['summary']
        self.author = info['author']
        self.email = info['author_email']
        self.compatibleReleases = releases
        self.latestRelease = releases['latest']

    def setFakedRemotePluginInfo(self):
        """Sets value for the attributes that need to be obtained from json file"""
        self.homePage = self.pluginSourceUrl
        self.compatibleReleases = {DEVEL_VERSION: {'upload_time': '   devel_mode'}}
        self.latestRelease = DEVEL_VERSION
        self.author = ' Developer mode'

    # ###################### Local data funcs ############################

    def setLocalPluginInfo(self):
        """Sets value for the attributes that can be obtained locally if the
        plugin is installed."""
        if self.isInstalled():

            metadata = {}
            # Take into account 2 cases here:
            # A.: plugin is a proper pipmodule and is installed as such
            # B.: Plugin is not yet a pipmodule but a local folder.
            try:
                package = pkg_resources.get_distribution(self.pipName)
                keys = ['Name', 'Version', 'Summary', 'Home-page', 'Author',
                        'Author-email']
                pattern = r'(.*): (.*)'

                for line in package._get_metadata(package.PKG_INFO):
                    match = re.match(pattern, line)
                    if match:
                        key = match.group(1)
                        if key in keys:
                            metadata[key] = match.group(2)
                            keys.remove(key)
                            if not len(keys):
                                break

                self.pipVersion = metadata.get('Version', "")
                self.dirName = self.getDirName()
                self.binVersions = self.getBinVersions()

            except:
                # Case B: code local but not yet a pipmodule.
                pass

            if not self.remote:
                # only do this if we don't already have it from remote
                self.homePage = metadata.get('Home-page', "")
                self.summary = metadata.get('Summary', "")
                self.author = metadata.get('Author', "")
                self.email = metadata.get('Author-email', "")

    def getPluginClass(self):
        """ Tries to find the Plugin object."""
        pluginModule = self._getPlugin()

        if pluginModule is not None:
            pluginClass = pluginModule.Plugin
        else:
            print("Warning: couldn't find Plugin for %s" % self.pipName)
            print("Dirname: %s" % self.getDirName())
            pluginClass = None
        return pluginClass

    def getInstallenv(self, envArgs=None):
        """Reads the defineBinaries function from Plugin class and returns an
        Environment object with the plugin's binaries."""
        if envArgs is None:
            envArgs = dict()

        env = Environment(**envArgs)
        env.setDefault(False)

        plugin = self.getPluginClass()
        if plugin is not None:
            try:
                plugin.defineBinaries(env)
            except Exception as e:
                print("Couldn't get binaries definition of %s plugin: %s" % (self.name, e))
            return env
        else:
            return None

    def getBinVersions(self):
        """Get list with names of binaries of this plugin"""
        env = Environment()
        env.setDefault(False)
        defaultTargets = [target.getName() for target in env.getTargetList()]
        plugin = self.getPluginClass()
        if plugin is not None:
            try:
                plugin.defineBinaries(env)
            except Exception as e:
                print(
                    redStr("Error retrieving plugin %s binaries: " % self.name), e)
        binVersions = [target.getName() for target in env.getTargetList() if target.getName() not in defaultTargets]
        return binVersions

    def getDirName(self):
        """Get the name of the folder that contains the plugin code
           itself (e.g. to import the _plugin object.)"""
        # top level file is a file included in all pip packages that contains
        # the name of the package's top level directory
        try:
            return pkg_resources.get_distribution(self.pipName).get_metadata('top_level.txt').strip()
        except Exception as e:
            return None

    def printBinInfoStr(self):
        """Returns string with info of binaries installed to print in console
        with flag --help"""
        try:
            env = self.getInstallenv()

            return env.printHelp().split('\n', 1)[1]
        except IndexError as noBins:
            return " ".rjust(14) + "No binaries information defined.\n"
        except Exception as e:
            return " ".rjust(14) + "Error getting binaries info: %s" % \
                   str(e) + "\n"

    def getPluginName(self):
        """Return the plugin name"""
        return self.name

    def getPipName(self):
        """Return the plugin pip name"""
        return self.pipName

    def getPipVersion(self):
        """Return the plugin pip version"""
        return self.pipVersion

    def getSourceUrl(self):
        """Return the plugin source url"""
        return self.pluginSourceUrl

    def getHomePage(self):
        """Return the plugin Home page"""
        return self.homePage

    def getSummary(self):
        """Return the plugin summary"""
        return self.summary

    def getAuthor(self):
        """Return the plugin author"""
        return self.author

    def getReleaseDate(self, release):
        """Return the uploaded date from the release"""
        return self.compatibleReleases[release]['upload_time']

    def getLatestRelease(self):
        """Get the plugin latest release"""
        return self.latestRelease


class PluginRepository(object):

    def __init__(self, repoUrl=REPOSITORY_URL):
        self.repoUrl = repoUrl
        self.plugins = None

    @staticmethod
    def getBinToPluginDict():
        localPlugins = Domain.getPlugins()
        binToPluginDict = {}
        for p, pobj in localPlugins.items():
            pinfo = PluginInfo(name=p, plugin=pobj, remote=False)
            pbins = pinfo.getBinVersions()
            binToPluginDict.update({k: p for k in pbins})
            pbinsNoVersion = set([b.split('-', 1)[0] for b in pbins])
            binToPluginDict.update({k: p for k in pbinsNoVersion})
        return binToPluginDict

    def getPlugins(self, pluginList=None, getPipData=False):
        """Reads available plugins from self.repoUrl and returns a dict with
        PluginInfo objects. Params:
        - pluginList: A list with specific plugin pip-names we want to get.
        - getPipData: If true, each PluginInfo object will try to get the data
        of the plugin from pypi."""

        pluginsJson = {}
        if self.plugins is None:
            self.plugins = {}

        if os.path.isfile(self.repoUrl):
            with open(self.repoUrl) as f:
                pluginsJson = json.load(f)
            getPipData = False
        else:
            try:
                r = requests.get(self.repoUrl)
                getPipData = True
            except requests.ConnectionError as e:
                print("\nWARNING: Error while trying to connect with a server:\n"
                      "  > Please, check your internet connection!\n")
                print(e)
                return self.plugins
            if r.ok:
                pluginsJson = r.json()
            else:
                print("WARNING: Can't get Scipion's plugin list, the plugin "
                      "repository is not available")
                return self.plugins

        availablePlugins = pluginsJson.keys()

        if pluginList is None:
            targetPlugins = availablePlugins
        else:
            targetPlugins = set(availablePlugins).intersection(set(pluginList))
            if len(targetPlugins) < len(pluginList):
                wrongPluginNames = set(pluginList) - set(availablePlugins)
                print("WARNING - The following plugins didn't match available "
                      "plugin names:")
                print(" ".join(wrongPluginNames))
                print("You can see the list of available plugins with the following command:\n"
                      "scipion installp --help")

        for pluginName in targetPlugins:
            pluginsJson[pluginName].update(remote=getPipData)
            pluginInfo = PluginInfo(**pluginsJson[pluginName])
            if pluginInfo.getLatestRelease() != NULL_VERSION:
                self.plugins[pluginName] = pluginInfo

        return self.plugins

    def printPluginInfoStr(self, withBins=False, withUpdates=False):
        """Returns string to print in console which plugins are installed.
        - withBins: If true, will add binary info for the plugins installed
        - with Updates: If true, will check if the installed plugins have new
                    releases."""
        def ansi(n):
            """Return function that escapes text with ANSI color n."""
            return lambda txt: '\x1b[%dm%s\x1b[0m' % (n, txt)

        black, red, green, yellow, blue, magenta, cyan, white = map(ansi,
                                                                    range(30, 38))

        printStr = ""
        pluginDict = self.getPlugins(getPipData=withUpdates)
        if pluginDict:
            withBinsStr = "Installed plugins and their %s" % green("binaries") \
                if withBins else "Available plugins"
            printStr += ("%s: "
                         "([ ] not installed, [X] seems already installed)\n\n" % withBinsStr)
            keys = sorted(pluginDict.keys())
            for name in keys:
                plugin = pluginDict[name]
                if withBins and not plugin.isInstalled():
                    continue
                printStr += "{0:30} {1:10} [{2}]".format(name,
                                                            plugin.pipVersion,
                                                            'X' if plugin.isInstalled() else ' ')
                if withUpdates and plugin.isInstalled():
                    if plugin.latestRelease != plugin.pipVersion:
                        printStr += yellow('\t(%s available)' % plugin.latestRelease)
                printStr += "\n"
                if withBins:
                    printStr += green(plugin.printBinInfoStr())
        else:
            printStr = "List of available plugins in plugin repository inaccessible at this time."
        return printStr



#!/usr/bin/env python

import os
import sys

cleanAll = 'all' in sys.argv

python = 'python' in sys.argv


def cmd(command):
    print(command)
    os.system(command)


if cleanAll:
    cmd('rm -rf software/lib/* ')
    cmd('rm -rf software/bin/*  ')
    cmd('rm -rf software/include/* ')
    cmd('rm -rf software/man/* ')
    cmd('rm -rf software/share/* ')
    cmd('rm -rf software/tmp/*')
    cmd('rm -rf software/log/*')

elif python:
    cmd('rm -rf software/bin/python* ')
    cmd('rm -rf software/lib/python* ')
    cmd('rm -rf software/include/python* ')

# Clean xmipp, default for empty args.
else:
    for ext in ['so', 'os', 'o']:
        cmd('find software/em/xmipp -name "*.%s" -exec rm -rf {} \;' % ext)
# force pyc refresh even if .pyc's timestamp is not older than the corresponding .py's timestamp
# to trigger a recompilation.
cmd('find . -iname "*.pyc" -delete')



#!/usr/bin/env python

# **************************************************************************
# *
# * Authors:     J. Burguet Castell     (jburguet@cnb.csic.es)
# *
# * Unidad de Bioinformatica of Centro Nacional de Biotecnologia, CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************

"""
Show all the system packages on which scipion depends.
"""

import os
from subprocess import check_output, CalledProcessError
import argparse


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    add = parser.add_argument  # short notation
    add('dirs', metavar='DIR', nargs='+', help="directories with binaries")
    add('--packages', action='store_true', help="show system packages")
    add('--pkg-command', default='dpkg -S',
        help="command used to find which package a file belongs to")
    args = parser.parse_args()

    # Get all the library dependencies.
    neededLibs = set()
    for subdir in args.dirs:
        for dirpath, dirnames, filenames in os.walk(subdir):
            for fname in filenames:
                fpath = os.path.join(dirpath, fname)
                if isElf(fpath):
                    neededLibs.update(libDeps(fpath))

    # Show where they are in the system.
    existingLibs = {path: os.listdir(path)
                    for path in searchPaths() if os.path.exists(path)}

    shown = set()
    for lib in neededLibs:
        for libPath in existingLibs:
            if lib not in existingLibs[libPath]:
                continue
            if not args.packages:
                libFull = '%s/%s' % (libPath, lib)
                if libFull not in shown:
                    print(libFull)
                    shown.add(libFull)
            elif 'software/lib' not in libPath:
                command = '%s %s/%s' % (args.pkg_command, libPath, lib)
                try:
                    pkg = check_output(command.split()).split(':')[0].strip()
                    if pkg not in shown:
                        print(pkg)
                        shown.add(pkg)
                except CalledProcessError:
                    print('%s/%s -> no package found' % (libPath, lib))
            break
        else:
            print('%s -> not found in any path' % lib)


def libDeps(fpath):
    """Return set of libraries that the file at fpath links to"""
    libs = set()
    for line in check_output(['objdump', '-p', fpath]).splitlines():
        fields = line.split()
        if fields and fields[0] == 'NEEDED':
            libs.add(fields[-1])
    return libs

    
def isElf(fname):
    """Is fname a file in elf format?"""
    return os.path.isfile(fname) and open(fname).read(4) == '\x7fELF'


def searchPaths():
    """Return the list of paths were the libraries are searched in the system"""
    paths = []

    if 'LD_LIBRARY_PATH' in os.environ:
        paths += os.environ.get('LD_LIBRARY_PATH', '').split(':')
    
    for fname in os.listdir('/etc/ld.so.conf.d'):
        if fname.endswith('.conf'):
            for line in open('/etc/ld.so.conf.d/%s' % fname):
                if not line.startswith('#'):
                    paths.append(line.strip())
    paths = [x for x in paths if 'i386' not in x]  # FIXME: something nicer
    return paths


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# **************************************************************************
# *
# * Authors:     J.M. De la Rosa Trevin (delarosatrevin@scilifelab.se) [1]
# *
# * [1] SciLifeLab, Stockholm University
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************


import sys
from os.path import join, exists, dirname
import importlib
import inspect
import traceback
from collections import OrderedDict
from future.utils import iteritems

from pwem.protocols import (Prot3D, Prot2D, ProtParticles,
                            ProtMicrographs, ProtImport)
from pwem import Domain
from pyworkflow.protocol import Protocol
import pyworkflow.utils as pwutils

from scipion.install.plugin_funcs import PluginInfo


exitWithErrors = False


def usage(error=""):

    if error:
        error = "ERROR: %s\n" % error

    print("""%s
    Usage: scipion python scripts/inspect-plugins.py [PLUGIN-NAME] [info] [--showBase]
        This script loads all Scipion plugins found.
        If a PLUGIN-NAME is passed, it will inspect that plugin
        in more detail.
        'info' argument will print plugin summary,
        '-showBase' will print Base class protocols (hidden by default).
    """ % error)
    sys.exit(1)


def getSubmodule(plugin, name, subname):
    """ Return a tuple: (module, error)
    If module is None:
        1) if error is None is that the submodule does not exist
        2) if error is not None, it is an Exception raised when
        importing the submodule
    """

    try:
        m = importlib.import_module('%s.%s' % (name, subname))
        r = (m, None)
    except Exception as e:
        noModuleMsg = 'No module named \'%s.%s\'' % (name, subname)
        msg = str(e)
        moduleExists = (exists(join(dirname(plugin.__file__), "%s.py" % subName)) or
                        exists(join(dirname(plugin.__file__), subName)))
        r = (None, None if msg == noModuleMsg and not moduleExists else traceback.format_exc())
    return r


def getFirstLine(doc):
    """ Get the first non empty line from doc. """
    if doc:
        for lines in doc.split('\n'):
            l = lines.strip()
            if l:
                return l
    return ''


n = len(sys.argv)

if n > 4:
    usage("Incorrect number of input parameters")

if n == 1:  # List all plugins
    plugins = Domain.getPlugins()
    print("Plugins:")
    for k, v in iteritems(plugins):
        print("-", k)

    print("Objects")
    pwutils.prettyDict(Domain.getObjects())

    print("Protocols")
    pwutils.prettyDict(Domain.getProtocols())

    print("Viewers")
    pwutils.prettyDict(Domain.getViewers())


elif n == 2:
    if sys.argv[1] in ['-h', '--help', 'help']:
        usage()

    pluginName = sys.argv[1]
    plugin = Domain.getPlugin(pluginName)
    print("Plugin: %s" % pluginName)
    for subName in ['constants', 'convert', 'protocols',
                    'wizards', 'viewers', 'tests']:
        sub, error = getSubmodule(plugin, pluginName, subName)

        if sub is None:
            if error is None:
                msg = " missing"
            else:
                exitWithErrors = True
                msg = " error -> %s" % error

        else:
            msg = " loaded"

        print("   >>> %s: %s" % (subName, msg))

elif n > 2:
    if sys.argv[2] == 'info':
        pluginName = sys.argv[1]
        showBase = True if (n == 4 and sys.argv[3] == '--showBase') else False
        subclasses = {}
        emCategories = [('Imports', ProtImport),
                        ('Micrographs', ProtMicrographs),
                        ('Particles', ProtParticles),
                        ('2D', Prot2D),
                        ('3D', Prot3D)]

        plugin = Domain.getPlugin(pluginName)
        version = PluginInfo('scipion-em-%s' % pluginName).pipVersion
        bin = PluginInfo('scipion-em-%s' % pluginName).printBinInfoStr()
        print("Plugin name: %s, version: %s" % (pluginName, version))
        print("Plugin binaries: %s" % bin)

        # print bibtex
        bib, error2 = getSubmodule(plugin, pluginName, 'bibtex')
        if bib is None:
            if error2 is None:
                msg = " missing bibtex"
            else:
                exitWithErrors = True
                msg = " error -> %s" % error2
        else:
            print("Plugin references:")
            bibtex = pwutils.parseBibTex(bib.__doc__)

            for citeStr in bibtex:
                text = Protocol()._getCiteText(bibtex[citeStr])
                print(text)

        # print protocols
        sub, error = getSubmodule(plugin, pluginName, 'protocols')
        if sub is None:
            if error is None:
                msg = " missing protocols"
            else:
                exitWithErrors = True
                msg = " error -> %s" % error

        else:
            for name in dir(sub):
                attr = getattr(sub, name)
                if inspect.isclass(attr) and issubclass(attr, Protocol):
                    # Set this special property used by Scipion
                    attr._package = plugin
                    subclasses[name] = attr

        print("Plugin protocols:\n")
        print("%-35s %-35s %-10s %-s" % (
            'NAME', 'LABEL', 'CATEGORY', 'DESCRIPTION'))

        prots = OrderedDict(sorted(subclasses.items()))
        for prot in prots:
            label = prots[prot].getClassLabel()
            desc = getFirstLine(prots[prot].__doc__)
            cat = 'None'

            for c in emCategories:
                if issubclass(prots[prot], c[1]):
                    cat = c[0]
                if prots[prot].isBase():
                    cat = 'Base prot'

            # skip Base protocols if not requested
            if prots[prot].isBase() and not showBase:
                continue
            else:
                print("%-35s %-35s %-10s %-s" % (prot, label, cat, desc))

    else:
        usage("The last argument must be 'info'")

if exitWithErrors:
    sys.exit(1)
else:
    sys.exit(0)



#!/usr/bin/env python

import sys
import os
from os.path import dirname, realpath, join


SCIPION_HOME = dirname(dirname(dirname(realpath(__file__))))


def usage(error):
    print("""
    ERROR: %s

    Usage: scripts/tar.py TARNAME
        TARNAME: the type of tar that is going to be generated from 
        the scipion folder. Must be 'source' or 'linux64'.
    """ % error)
    sys.exit(1)


def _parseVersionDate():
    """ Parse the version and release date from scipion script. """
    scipionScript = open(join(SCIPION_HOME, 'scipion'))
    version, date = None, None

    for line in scipionScript:
        l = line.strip()
        if '__version__ =' in l:
            version = line.split("'")[1]
        elif '__releasedate__ =' in line:
            date = line.split("'")[1]
            break

    return version, date


if len(sys.argv) != 2:
    usage("Incorrect number of input parameters")


version, date = _parseVersionDate()
label = sys.argv[1]
if label == 'source':
    excludeTgz = ''
elif label == 'linux64':
    excludeTgz = "--exclude='*.tgz' --exclude='*.h' --exclude='*.cpp' --exclude='*.java'"

else:
    usage("incorrect TARNAME'")

args = {'label': label,
        'version': version,
        'date': date,
        'excludeTgz': excludeTgz
        }

# We will generate the tar from the same level of scipion git folder
cwd = os.getcwd()
os.chdir(dirname(SCIPION_HOME))

cmdStr = """ tar czf scipion_%(version)s_%(date)s_%(label)s.tgz \\
--exclude=.git --exclude='*.o' --exclude='*.os' --exclude='*pyc' \\
--exclude='*.mrc' --exclude='*.stk' --exclude='*.gz' %(excludeTgz)s \\
--exclude='software/tmp/*' --exclude='*.scons*' --exclude='config/*.conf' scipion
"""

cmd = cmdStr % args

print(cmd)
os.system(cmd)

# Restore current working dir
os.chdir(cwd)



# **************************************************************************
# *
# * Authors: J.M. De la Rosa Trevin (delarosatrevin@scilifelab.se) [1]
# *
# * [1] SciLifeLab, Stockholm University
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************

import os
import platform
import sys
import time
from glob import glob
from os.path import join, exists, islink, abspath
from subprocess import STDOUT, call

from pyworkflow import Config
import pwem


try:
    unicode = unicode
except NameError:  # 'unicode' is undefined, must be Python 3
    unicode = str
    basestring = (str, bytes)

# Then we get some OS vars
MACOSX = (platform.system() == 'Darwin')
WINDOWS = (platform.system() == 'Windows')
LINUX = (platform.system() == 'Linux')
VOID_TGZ="void.tgz"


def ansi(n):
    """Return function that escapes text with ANSI color n."""
    return lambda txt: '\x1b[%dm%s\x1b[0m' % (n, txt)


black, red, green, yellow, blue, magenta, cyan, white = map(ansi, range(30, 38))
# We don't take them from pyworkflow.utils because this has to run
# with all python versions (and so it is simplified).


def progInPath(prog):
    """ Is program prog in PATH? """
    for base in os.environ.get('PATH', '').split(os.pathsep):
        if exists('%s/%s' % (base, prog)):
            return True
    return False


def checkLib(lib, target=None):
    """ See if we have library lib """
    try:
        ret = call(['pkg-config', '--cflags', '--libs', lib],
                   stdout=open(os.devnull, 'w'), stderr=STDOUT)
        if ret != 0:
            raise OSError
    except OSError:
        try:
            ret = call(['%s-config' % lib, '--cflags'])
            if ret != 0:
                raise OSError
        except OSError:
            print("""
  ************************************************************************
    Warning: %s not found. Please consider installing it first.
  ************************************************************************

Continue anyway? (y/n)""" % lib)
            if input().upper() != 'Y':
                sys.exit(2)
    # TODO: maybe write the result of the check in
    # software/log/lib_...log so we don't check again if we already said "no"


class Command:
    def __init__(self, env, cmd, targets=None,  **kwargs):
        self._env = env
        self._cmd = cmd

        if targets is None:
            self._targets = []
        elif isinstance(targets, basestring):
            self._targets = [targets]
        else:
            self._targets = targets

        self._cwd = kwargs.get('cwd', None)
        self._out = kwargs.get('out', None)
        self._always = kwargs.get('always', False)
        self._environ = kwargs.get('environ', None)

    def _existsAll(self):
        """ Return True if all targets exist. """
        for t in self._targets:
            if not glob(t):
                return False
        return True

    def execute(self):
        if not self._always and self._targets and self._existsAll():
            print("  Skipping command: %s" % cyan(self._cmd))
            print("  All targets exist.")
        else:
            cwd = os.getcwd()
            if self._cwd is not None:
                if not self._env.showOnly:
                    os.chdir(self._cwd)
                print(cyan("cd %s" % self._cwd))
    
            # Actually allow self._cmd to be a list or a
            # '\n'-separated list of commands, and run them all.
            if isinstance(self._cmd, basestring):
                cmds = self._cmd.split('\n')  # create list of commands
            elif callable(self._cmd):
                cmds = [self._cmd]  # a function call
            else:
                cmds = self._cmd  # already a list of whatever
    
            for cmd in cmds:
                if self._out is not None:
                    cmd += ' > %s 2>&1' % self._out
                    # TODO: more general, this only works for bash.
        
                print(cyan(cmd))
        
                if self._env.showOnly:
                    continue  # we don't really execute the command here
        
                if callable(cmd):  # cmd could be a function: call it
                    cmd()
                else:  # if not, it's a command: make a system call
                    call(cmd, shell=True, env=self._environ,
                         stdout=sys.stdout, stderr=sys.stderr)

            # Return to working directory, useful when we change dir
            # before executing the command.
            os.chdir(cwd)
            if not self._env.showOnly:
                for t in self._targets:
                    assert glob(t), ("target '%s' not built (after "
                                     "running '%s')" % (t, cmd))

    def __str__(self):
        return "Command: %s, targets: %s" % (self._cmd, self._targets)


class Target:
    def __init__(self, env, name, *commands, **kwargs):
        self._env = env
        self._name = name
        self._default = kwargs.get('default', False)
        self._always = kwargs.get('always', False)  # Adding always here to allow getting to Commands where always=True
        self._commandList = list(commands)  # copy the list/tuple of commands
        self._finalCommands = []  # their targets will be used to check if we need to re-build
        self._deps = []  # names of dependency targets

    def getCommands(self):
        return self._commandList

    def addCommand(self, cmd, **kwargs):
        if isinstance(cmd, Command):
            c = cmd
        else:
            c = Command(self._env, cmd, **kwargs)
        self._commandList.append(c)

        if kwargs.get('final', False):
            self._finalCommands.append(c)
        return c

    def addDep(self, dep):
        self._deps.append(dep)

    def getDeps(self):
        return self._deps

    def _existsAll(self):
        for c in self._finalCommands:
            if not c._existsAll():
                return False
        return True

    def isDefault(self):
        return self._default

    def setDefault(self, default):
        self._default = default

    def getName(self):
        return self._name

    def execute(self):
        t1 = time.time()

        print(green("Building %s ..." % self._name))
        if not self._always and self._existsAll():
            print("  All targets exist, skipping.")
        else:
            for command in self._commandList:
                command.execute()

        if not self._env.showOnly:
            dt = time.time() - t1
            if dt < 60:
                print(green('Done (%.2f seconds)' % dt))
            else:
                print(green('Done (%d m %02d s)' % (dt / 60, int(dt) % 60)))

    def __str__(self):
        return self._name


class Environment:

    def __init__(self, **kwargs):
        self._targetDict = {}
        self._targetList = []
        # We need a targetList which has the targetDict.keys() in order
        # (OrderedDict is not available in python < 2.7)

        self._packages = {}  # dict of available packages (to show in --help)

        self._args = kwargs.get('args', [])
        self.showOnly = '--show' in self._args

        # Find if the -j arguments was passed to get the number of processors
        if '-j' in self._args:
            j = self._args.index('-j')
            self._processors = int(self._args[j+1])
        else:
            self._processors = 1
            
        if LINUX:
            self._libSuffix = 'so'  # Shared libraries extension name
        else:
            self._libSuffix = 'dylib'

        self._downloadCmd = ('wget -nv -c -O %(tar)s.part %(url)s\n'
                             'mv -v %(tar)s.part %(tar)s')
        # Removed the z: "The tar command auto-detects compression type and extracts the archive"
        # From https://linuxize.com/post/how-to-extract-unzip-tar-bz2-file/#extracting-tarbz2-file
        self._tarCmd = 'tar -xf %s'
        self._pipCmd = kwargs.get('pipCmd', 'pip install %s==%s')

    def getLibSuffix(self):
        return self._libSuffix
    
    def getProcessors(self):
        return self._processors
    
    @staticmethod
    def getSoftware(*paths):
        return os.path.join(Config.SCIPION_SOFTWARE, *paths)

    @staticmethod
    def getLibFolder(*paths):
        return Environment.getSoftware("lib", *paths)

    @staticmethod
    def getPython():
        return sys.executable

    # Pablo: A quick search didn't find usages.
    # @staticmethod
    # def getPythonFolder():
    #     return Environment.getLibFolder() + '/python2.7'

    @staticmethod
    def getPythonPackagesFolder():
        # This does not work on MAC virtual envs
        # import site
        # return site.getsitepackages()[0]

        from distutils.sysconfig import get_python_lib
        return get_python_lib()

    @staticmethod
    def getIncludeFolder():
        return Environment.getSoftware('include')

    def getLib(self, name):

        return Environment.getLibFolder('lib%s.%s' % (name, self._libSuffix))

    @staticmethod
    def getBinFolder(*paths):
        return  os.path.join(mkdir(Environment.getSoftware('bin')), *paths)

    @staticmethod
    def getBin(name):
        return  Environment.getBinFolder(name)

    @staticmethod
    def getTmpFolder():
        return  mkdir(Environment.getSoftware('tmp'))

    @staticmethod
    def getLogFolder(*path):
        return os.path.join(mkdir(Environment.getSoftware('log')),*path)

    @staticmethod
    def getEmFolder():
        return mkdir(pwem.Config.EM_ROOT)

    @staticmethod
    def getEm(name):
        return '%s/%s' % (Environment.getEmFolder(), name)

    def getTargetList(self):
        return self._targetList

    def addTarget(self, name, *commands, **kwargs):

        if name in self._targetDict:
            raise Exception("Duplicated target '%s'" % name)

        t = Target(self, name, *commands, **kwargs)
        self._targetList.append(t)
        self._targetDict[name] = t

        return t

    def addTargetAlias(self, name, alias):
        """ Add an alias to an existing target.
        This function will be used for installing the last version of each
        package.
        """
        if name not in self._targetDict:
            raise Exception("Can't add alias, target name '%s' not found. "
                            % name)

        self._targetDict[alias] = self._targetDict[name]
    
    def getTarget(self, name):
        return self._targetDict[name]
    
    def hasTarget(self, name):
        return name in self._targetDict
    
    def getTargets(self):
        return self._targetList

    def _addTargetDeps(self, target, deps):
        """ Add the dependencies to target.
        Check that each dependency correspond to a previous target.
        """
        for d in deps:
            if isinstance(d, str):
                targetName = d
            elif isinstance(d, Target):
                targetName = d.getName()
            else:
                raise Exception("Dependencies should be either string or "
                                "Target, received: %s" % d)

            if targetName not in self._targetDict:
                raise Exception("Dependency '%s' does not exists. " % targetName)

            target.addDep(targetName)

    def _addDownloadUntar(self, name, **kwargs):
        """ Buid a basic target and add commands for Download and Untar.
        This is the base for addLibrary, addModule and addPackage.
        """
        # Use reasonable defaults.
        tar = kwargs.get('tar', '%s.tgz' % name)
        urlSuffix = kwargs.get('urlSuffix', 'external')
        url = kwargs.get('url', '%s/%s/%s' % (Config.SCIPION_URL_SOFTWARE, urlSuffix, tar))
        downloadDir = kwargs.get('downloadDir', self.getTmpFolder())
        buildDir = kwargs.get('buildDir',
                              tar.rsplit('.tar.gz', 1)[0].rsplit('.tgz', 1)[0])
        targetDir = kwargs.get('targetDir', buildDir)

        createBuildDir = kwargs.get('createBuildDir', False)

        deps = kwargs.get('deps', [])
        
        # Download library tgz
        tarFile = join(downloadDir, tar)
        buildPath = join(downloadDir, buildDir)
        targetPath = join(downloadDir, targetDir)
        
        t = self.addTarget(name, default=kwargs.get('default', True))
        self._addTargetDeps(t, deps)
        t.buildDir = buildDir
        t.buildPath = buildPath
        t.targetPath = targetPath

        # check if tar exists and has size >0 so that we can download again
        if os.path.isfile(tarFile) and os.path.getsize(tarFile) == 0:
            os.remove(tarFile)

        if url.startswith('file:'):
            t.addCommand('ln -s %s %s' % (url.replace('file:', ''), tar),
                         targets=tarFile,
                         cwd=downloadDir)
        else:
            t.addCommand(self._downloadCmd % {'tar': tarFile, 'url': url},
                         targets=tarFile)

        if createBuildDir:
            tarCmd = '{0} -C {1}'.format(self._tarCmd % tar, buildDir)
            t.addCommand('mkdir %s' % buildPath,
                         targets=[buildPath],
                         cwd=downloadDir)
        else:
            tarCmd = self._tarCmd % tar


        finalTarget = join(downloadDir, kwargs.get('target', buildDir))
        t.addCommand(tarCmd,
                     targets=finalTarget,
                     cwd=downloadDir)
        
        return t          
         
    def addLibrary(self, name, **kwargs):
        """Add library <name> to the construction process.

        Checks that the needed programs are in PATH, needed libraries
        can be found, downloads the given url, untars the resulting
        tar file, configures the library with the given flags,
        compiles it (in the given buildDir) and installs it.

        If default=False, the library will not be built.

        Returns the final targets, the ones that Make will create.

        """
        configTarget = kwargs.get('configTarget', 'Makefile')
        configAlways = kwargs.get('configAlways', False)
        flags = kwargs.get('flags', [])
        targets = kwargs.get('targets', [self.getLib(name)])
        clean = kwargs.get('clean', False)  # Execute make clean at the end??
        cmake = kwargs.get('cmake', False)  # Use cmake instead of configure??
        default = kwargs.get('default', True)
        neededProgs = kwargs.get('neededProgs', [])
        libChecks = kwargs.get('libChecks', [])

        if default or name in sys.argv[2:]:
            # Check that we have the necessary programs and libraries in place.
            for prog in neededProgs:
                assert progInPath(prog), ("Cannot find necessary program: %s\n"
                                          "Please install and try again" % prog)
            for lib in libChecks:
                checkLib(lib)

        # If passing a command list (of tuples (command, target)) those actions
        # will be performed instead of the normal ./configure / cmake + make
        commands = kwargs.get('commands', [])

        t = self._addDownloadUntar(name, **kwargs)
        configDir = kwargs.get('configDir', t.buildDir)

        configPath = join(self.getTmpFolder(), configDir)
        makeFile = '%s/%s' % (configPath, configTarget)
        prefix = abspath(Environment.getSoftware())

        # If we specified the commands to run to obtain the target,
        # that's the only thing we will do.
        if commands:
            for cmd, tgt in commands:
                t.addCommand(cmd, targets=tgt, final=True)
                # Note that we don't use cwd=t.buildDir, so paths are
                # relative to SCIPION_HOME.
            return t

        # If we didn't specify the commands, we can either compile
        # with autotools (so we have to run "configure") or cmake.

        environ = os.environ.copy()
        for envVar, value in [('CPPFLAGS', '-I%s/include' % prefix),
                              ('LDFLAGS', '-L%s/lib' % prefix)]:
            environ[envVar] = '%s %s' % (value, os.environ.get(envVar, ''))

        if not cmake:
            flags.append('--prefix=%s' % prefix)
            flags.append('--libdir=%s/lib' % prefix)
            t.addCommand('./configure %s' % ' '.join(flags),
                         targets=makeFile, cwd=configPath,
                         out=self.getLogFolder('%s_configure.log' % name),
                         always=configAlways, environ=environ)
        else:
            assert progInPath('cmake') or 'cmake' in sys.argv[2:], \
                "Cannot run 'cmake'. Please install it in your system first."

            flags.append('-DCMAKE_INSTALL_PREFIX:PATH=%s .' % prefix)
            t.addCommand('cmake %s' % ' '.join(flags),
                         targets=makeFile, cwd=configPath,
                         out=self.getLogFolder('%s_cmake.log' % name),
                         environ=environ)

        t.addCommand('make -j %d' % self._processors,
                     cwd=t.buildPath,
                     out=self.getLogFolder('%s_make.log' %  name))

        t.addCommand('make install',
                     targets=targets,
                     cwd=t.buildPath,
                     out=self.getLogFolder('%s_make_install.log' % name),
                     final=True)

        if clean:
            t.addCommand('make clean',
                         cwd=t.buildPath,
                         out=self.getLogFolder('%s_make_clean.log' % name))
            t.addCommand('rm %s' % makeFile)

        return t

    def addPipModule(self, name, version="", pipCmd=None,
                     target=None, default=True, deps=[]):
        """Add a new module to our built Python .
        Params in kwargs:
            name: pip module name
            version: module version - must be specified to prevent undesired updates.
            default: True if this module is build by default.
        """

        target = name if target is None else target
        pipCmd = pipCmd or self._pipCmd % (name, version)
        t = self.addTarget(name, default=default, always=True)  # we set always=True to let pip decide if updating

        # Add the dependencies
        defaultDeps = []

        self._addTargetDeps(t, defaultDeps + deps)

        t.addCommand(pipCmd,
                     final=True,
                     targets="%s/%s" % (self.getPythonPackagesFolder(), target),
                     always=True  # execute pip command always. Pip will handle target existence
                     )

        return t

    def addPackage(self, name, **kwargs):
        """ Download a package tgz, untar it and create a link in software/em.
        Params in kwargs:
            tar: the package tar file, by default the name + .tgz. Pass None or VOID_TGZ if there is no tar file.
            commands: a list with actions to be executed to install the package
        """
        # Add to the list of available packages, for reference (used in --help).
        neededProgs = kwargs.get('neededProgs', [])

        if name in sys.argv[2:]:
            # Check that we have the necessary programs in place.
            for prog in neededProgs:
                assert progInPath(prog), ("Cannot find necessary program: %s\n"
                                          "Please install and try again" % prog)

        if name not in self._packages:
            self._packages[name] = []

        # Get the version from the kwargs
        if 'version' in kwargs:
            version = kwargs['version']
            extName = self._getExtName(name, version)
        else:
            version = ''
            extName = name

        self._packages[name].append((name, version))

        environ = (self.updateCudaEnviron(name)
                   if kwargs.get('updateCuda', False) else None)

        # Set environment
        variables = kwargs.get('vars', {})
        if variables:
            environ = {} if environ is None else environ
            environ.update(variables)

        # We reuse the download and untar from the addLibrary method
        # and pass the createLink as a new command 
        tar = kwargs.get('tar', '%s.tgz' % extName)

        # If tar is None or void.tgz
        if tar is None or tar == VOID_TGZ:
            tar = VOID_TGZ
            kwargs["buildDir"] = extName
            kwargs["createBuildDir"] = True

        buildDir = kwargs.get('buildDir',
                          tar.rsplit('.tar.gz', 1)[0].rsplit('.tgz', 1)[0])
        targetDir = kwargs.get('targetDir', buildDir)

        libArgs = {'downloadDir': self.getEmFolder(),
                   'urlSuffix': 'em',
                   'default': False}  # This will be updated with value in kwargs
        libArgs.update(kwargs)

        target = self._addDownloadUntar(extName, **libArgs)
        commands = kwargs.get('commands', [])
        for cmd, tgt in commands:
            if isinstance(tgt, basestring):
                tgt = [tgt]

            # Take all package targets relative to package build dir
            normTgt = []
            for t in tgt:
                # Check for empty targets and warn about them
                if not t:
                    print("WARNING: Target empty for command %s" % cmd)

                normTgt.append(join(target.targetPath, t))

            target.addCommand(cmd, targets=normTgt, cwd=target.buildPath,
                              final=True, environ=environ)
        
        target.addCommand(Command(self, Link(extName, targetDir),
                                  targets=[self.getEm(extName),
                                           self.getEm(targetDir)],
                                  cwd=self.getEm('')),
                          final=True)
        
        # Create an alias with the name for that version
        # this imply that the last package version added will be
        # the one installed by default, so the last versions should
        # be the last ones to be inserted
        self.addTargetAlias(extName, name)

        return target

    def _showTargetGraph(self, targetList):
        """ Traverse the targets taking into account
        their dependences and print them in DOT format.
        """
        print('digraph libraries {')
        for tgt in targetList:
            deps = tgt.getDeps()
            if deps:
                print('\n'.join("  %s -> %s" % (tgt, x) for x in deps))
            else:
                print("  %s" % tgt)
        print('}')

    def _showTargetTree(self, targetList, maxLevel=-1):
        """ Print the tree of dependencies for the given targets,
        up to a depth level of maxLevel (-1 for unlimited).
        """
        # List of (indent level, target)
        nodes = [(0, tgt) for tgt in targetList[::-1]]
        while nodes:
            lvl, tgt = nodes.pop()
            print("%s- %s" % ("  " * lvl, tgt))
            if maxLevel != -1 and lvl >= maxLevel:
                continue
            nodes.extend((lvl + 1, self._targetDict[x]) for x in tgt.getDeps())

    def _executeTargets(self, targetList):
        """ Execute the targets in targetList, running all their
        dependencies first.
        """
        executed = set()  # targets already executed
        exploring = set()  # targets whose dependencies we are exploring
        targets = targetList[::-1]
        while targets:
            tgt = targets.pop()
            if tgt.getName() in executed:
                continue
            deps = tgt.getDeps()
            if set(deps) - executed:  # there are dependencies not yet executed
                if tgt.getName() in exploring:
                    raise RuntimeError("Cyclic dependency on %s" % tgt)
                exploring.add(tgt.getName())
                targets.append(tgt)
                targets.extend(self._targetDict[x] for x in deps)
            else:
                tgt.execute()
                executed.add(tgt.getName())
                exploring.discard(tgt.getName())

    @staticmethod
    def _getExtName(name, version):
        """ Return folder name for a given package-version """
        return '%s-%s' % (name, version)

    def _isInstalled(self, name, version):
        """ Return true if the package-version seems to be installed. """
        pydir = self.getPythonPackagesFolder()
        extName = self._getExtName(name, version)
        return (exists(join(self.getEmFolder(), extName)) or
                extName in [x[:len(extName)] for x in os.listdir(pydir)])

    def printHelp(self):
        printStr = ""
        if self._packages:
            printStr = ("Available binaries: "
                        "([ ] not installed, [X] seems already installed)\n\n")

            keys = sorted(self._packages.keys())
            for k in keys:
                pVersions = self._packages[k]
                printStr += "{0:25}".format(k)
                for name, version in pVersions:
                    installed = self._isInstalled(name, version)
                    printStr += '{0:8}[{1}]{2:5}'.format(version, 'X' if installed else ' ', ' ')
                printStr += '\n'
        return printStr

    def execute(self):
        if '--help' in self._args:
            print(self.printHelp())
            return

        # Check if there are explicit targets and only install
        # the selected ones, ignore starting with 'xmipp'
        cmdTargets = [a for a in self._args
                      if a[0].isalpha()]
        if cmdTargets:
            # Check that they are all command targets
            for t in cmdTargets:
                if t not in self._targetDict:
                    raise RuntimeError("Unknown target: %s" % t)
            # Grab the targets passed in the command line
            targetList = [self._targetDict[t] for t in cmdTargets]
        else:
            # use all targets marked as default
            targetList = [t for t in self._targetList if t.isDefault()]

        if '--show-tree' in self._args:
            if '--dot' in self._args:
                self._showTargetGraph(targetList)
            else:
                self._showTargetTree(targetList)
        else:
            self._executeTargets(targetList)
        
    def updateCudaEnviron(self, package):
        """ Update the environment adding CUDA_LIB and/or CUDA_BIN to support
        packages that uses CUDA.
        package: package that needs CUDA to compile.
        """
        packUpper = package.upper()
        cudaLib = os.environ.get(packUpper + '_CUDA_LIB')
        cudaBin = os.environ.get(packUpper + '_CUDA_BIN')
    
        if cudaLib is None:
            cudaLib = pwem.Config.CUDA_LIB
            cudaBin = pwem.Config.CUDA_BIN

        environ = os.environ.copy()

        # If there isn't any CUDA in the environment
        if cudaLib is None and cudaBin is None:
            # Exit ...do not update the environment
            return environ

        elif cudaLib is not None and cudaBin is None:
            raise Exception("CUDA_LIB (or %s_CUDA_LIB) is defined, but not "
                            "CUDA_BIN (or %s_CUDA_BIN), please excecute "
                            "scipion config --update" % (packUpper, packUpper))
        elif cudaBin is not None and cudaLib is None:
            raise Exception("CUDA_BIN (or %s_CUDA_BIN) is defined, but not "
                            "CUDA_LIB (or %s_CUDA_LIB), please excecute "
                            "scipion config --update" % (packUpper, packUpper))
        elif os.path.exists(cudaLib) and os.path.exists(cudaBin):
            environ.update({'LD_LIBRARY_PATH': cudaLib + ":" +
                                               environ['LD_LIBRARY_PATH']})
            environ.update({'PATH': cudaBin + ":" + environ['PATH']})

        return environ

    def setDefault(self, default):
        """Set default values of all packages to the passed parameter"""
        for t in self._targetList:
            t.setDefault(default)

    def getPackages(self):
        """Return all plugin packages"""
        return self._packages

    def hasPackage(self, name):
        """ Returns true if it has the package"""
        return name in self._packages

    def getPackage(self, name):
        return  self._packages.get(name, None)

class Link:
    def __init__(self, packageLink, packageFolder):
        self._packageLink = packageLink
        self._packageFolder = packageFolder
        
    def __call__(self):
        self.createPackageLink(self._packageLink, self._packageFolder)
        
    def __str__(self):
        return "Link '%s -> %s'" % (self._packageLink, self._packageFolder)
        
    def createPackageLink(self, packageLink, packageFolder):
        """ Create a link to packageFolder in packageLink, validate
        that packageFolder exists and if packageLink exists it is 
        a link.
        This function is supposed to be executed in software/em folder.
        """
        linkText = "'%s -> %s'" % (packageLink, packageFolder)
        
        if not exists(packageFolder):
            print(red("Creating link %s, but '%s' does not exist!!!\n"
                      "INSTALLATION FAILED!!!" % (linkText, packageFolder)))
            sys.exit(1)
    
        if exists(packageLink):
            if islink(packageLink):
                os.remove(packageLink)
            else:
                print(red("Creating link %s, but '%s' exists and is not a link!!!\n"
                          "INSTALLATION FAILED!!!" % (linkText, packageLink)))
                sys.exit(1)
    
        os.symlink(packageFolder, packageLink)
        print("Created link: %s" % linkText)

def mkdir(path):
    """ Creates a folder if it does not exists"""
    if not exists(path):
        os.makedirs(path)
    return path


#!/usr/bin/env python

import os

RELEASE_ORI = '1.0.1'
RELEASE_END = '1.1'

EMPACK_DIRS_CONVERT = [
    {RELEASE_END: 'ctffind-3.5', RELEASE_ORI: 'ctffind_V3.5'},
    {RELEASE_END: 'ctffind4-4.0.15', RELEASE_ORI: 'ctffind_V4.0.15'},
    {RELEASE_END: 'eman-2.11', RELEASE_ORI: 'eman2.11.linux64'},
    {RELEASE_END: 'eman-2.12', RELEASE_ORI: 'eman2.12.linux64'},
    {RELEASE_END: 'frealign-9.07', RELEASE_ORI: 'frealign_v9.07'},
    {RELEASE_END: 'motioncorr-2.1', RELEASE_ORI: 'motioncorr_v2.1'},
    {RELEASE_END: 'relion-1.4f', RELEASE_ORI: 'relion-1.4_float'},
    {RELEASE_END: 'resmap-1.1.5s2', RELEASE_ORI: 'resmap-1.1.5-s2'},
    {RELEASE_END: 'simple-2.1', RELEASE_ORI: 'simple2'},
    {RELEASE_END: 'spider-21.13', RELEASE_ORI: 'spider-web-21.13'},
    {RELEASE_END: 'summovie-1.0.2', RELEASE_ORI: 'summovie_1.0.2'},
    {RELEASE_END: 'unblur-1.0.15', RELEASE_ORI: 'unblur_1.0_150529'}
]

print("This script is going to upgrade EM packages names from v1.0.1 to v1.1.0")

# Script folder
scd = os.path.dirname(__file__)
# Software EM folder
semd = os.path.join(scd, '..', 'software', 'em')

print("Script folder %s" % scd)
print("SEM folder %s" % semd)

# Move to Software EM folder
os.chdir(semd)

for emp in EMPACK_DIRS_CONVERT:
    print("Check if %s exists" % emp[RELEASE_ORI])
    if os.path.exists(emp[RELEASE_ORI]):
        print("Create symlink between %s and %s" % (emp[RELEASE_END], emp[RELEASE_ORI]))
        os.symlink(emp[RELEASE_ORI], emp[RELEASE_END])

# Move to back Scipion home
os.chdir("../..")

print("EM packages installed after the upgrade")
os.system('./scipion install --help')



#!/usr/bin/python

# Watch out: not "/usr/bin/env python", because we will overwrite the
# python in SCIPION_HOME/software/bin and we cannot use itself to run it.

# **************************************************************************
# *
# * Authors:     J. Burguet Castell     (jburguet@cnb.csic.es)
# *
# * Unidad de Bioinformatica of Centro Nacional de Biotecnologia, CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************

#
# To use after compiling with something like:
#   LDFLAGS=-Wl,-rpath,REPLACE_ME_WITH_FUNNY_ ./scipion install 
#

"""
Replaces in-place a text in binary files.
"""

import sys
import os
from tempfile import TemporaryFile


def main():
    if len(sys.argv) < 2:
        sys.exit('usage: %s <dir1> [<dir2> [...]]' % sys.argv[0])

    replacements = 0
    for subdir in sys.argv[1:]:
        print('Replacing RPATH in elf files under %s ...' % subdir)
        for dirpath, dirnames, filenames in os.walk(subdir):
            for fname in filenames:
                fpath = os.path.join(dirpath, fname)
                if isElf(fpath):
                    replacements += replace(fpath)

    print('Total replacements: %d' % replacements)
    if replacements == 0:
        print('Warning: You probably did not really compile. Clean and repeat.')


def isElf(fname):
    """Is fname a file in elf format?"""
    return os.path.isfile(fname) and open(fname).read(4) == '\x7fELF'


def replace(fname,
            txt_from='REPLACE_ME_WITH_FUNNY_',
            txt_to='$ORIGIN:$ORIGIN/../lib'):
    """Change txt_from to txt_to in binary file fname. Return 1 if changed."""

    # First add write permission to the file if it does not have it.
    if os.stat(fname).st_mode & 0o200 == 0:  # mode has no write permission
        print('Adding write permission to %s ...' % fname)
        os.chmod(fname,  os.stat(fname).st_mode + 0o200)

    changed = 0
    with TemporaryFile('wb+') as fout:
        with open(fname, 'rb') as fin:
            for line in fin:
                if txt_from in line:
                    print('Replacing in %s' % fname)
                    line = line.replace(txt_from, txt_to)
                    changed += 1
                fout.write(line)
        fout.seek(0)
        open(fname, 'wb').writelines(fout)  # overwrite fname

    return changed


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# **************************************************************************
# *
# * Authors:    J. M. De la Rosa Trevin (delarosatrevin@scilifelab.se) [1]
# *             I. Foche Perez (ifoche@cnb.csic.es) [2]
# *             P. Conesa (pconesa@cnb.csic.es) [2]
# *
# *  [1] SciLifeLab, Stockholm University
# *  [2] Unidad de Bioinformatica of Centro Nacional de Biotecnologia, CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
"""
Main entry point to scipion. It launches the gui, tests, etc.
"""
import sys
import os
from os.path import join, exists, expanduser, expandvars

from configparser import ConfigParser  # Python 3
from scipion.constants import *
from scipion.utils import (getScipionHome, getInstallPath,
                           getScriptsPath, getTemplatesPath, getModuleFolder)
from .scripts.config import getConfigPathFromConfigFile, PROTOCOLS, HOSTS
from scipion.constants import MODE_UPDATE
from scipion import __version__

__nickname__ = DEVEL
__releasedate__ = ''


# *********************  Helper functions *****************************
def getVersion(long=True):
    if long:
        return "v%s (%s) %s" % (__version__, __releasedate__, __nickname__)
    else:
        return __version__


def printVersion():
    """ Print Scipion version """
    # Print the version and some more info
    print('Scipion %s' % getVersion())


def config2Dict(configFile, varDict):
    """ Loads a config file if exists and populates a dictionary
    overwriting the keys.
    """
    # If config file exists
    if exists(configFile):
        #read the file
        config = ConfigParser()
        config.optionxform = str  # keep case (stackoverflow.com/questions/1611799)
        config.read(configFile)

        # For each section
        for sectionName, section in config.items():
            for variable, value in section.items():
                # Expanding user and avoiding comments
                cleanValue = value.split('#')[0]
                varDict[variable] = expandvars(cleanValue).strip()

    return varDict

def envOn(varName):
    value = os.environ.get(varName, '').lower()
    return value in ['1', 'true', 'on', 'yes']

def getMode():
    """ :returns the mode scipion has to be launched """
    return MODE_MANAGER if len(sys.argv) == 1 else sys.argv[1]

# Auxiliary functions to run commands in our environment, one of our
# scripts, or one of our "apps"
def runCmd(cmd, args=''):
    """ Runs ANY command with its arguments"""
    if isinstance(args, list):
        args = ' '.join('"%s"' % x for x in args)

    cmd = '%s %s' % (cmd, args)

    os.environ.update(VARS)
    # sys.stdout.write(">>>>> %s\n" % cmd)
    result = os.system(cmd)
    if not -256 < result < 256:
        result = 1  # because if not, 256 is confused with 0 !
    sys.exit(result)


# The following functions require a working SCIPION_PYTHON
def runScript(scriptCmd, args='', chdir=True):
    """"Runs a PYTHON script appending the profiling prefix if ON"""
    if chdir:
        os.chdir(Vars.SCIPION_HOME)

    if envOn('SCIPION_PROFILE'):
        profileStr = '-m cProfile -o output.profile'
    else:
        profileStr = ''
    cmd = '%s %s %s' % (Vars.SCIPION_PYTHON, profileStr, scriptCmd)
    runCmd(cmd, args)


def runApp(app, args='', chdir=True):
    """Runs an app provided by pyworkflow"""
    runScript(join(Vars.PW_APPS, app), args=args, chdir=chdir)


# ***************** END FUNCTIONS *****************************************

# Get Scipion home
scipionHome = getScipionHome()

# ***************** CONFIGURATION  FILES RESOLUTION ************************
# Default values for configuration files.
scipionConfig = join(scipionHome, 'config', 'scipion.conf')
scipionLocalConfig = expanduser(os.environ.get('SCIPION_LOCAL_CONFIG',
                                             '~/.config/scipion/scipion.conf'))

# Allow the user to override them (and remove them from sys.argv).
while len(sys.argv) > 2 and sys.argv[1].startswith('--'):
    arg = sys.argv.pop(1)
    value = sys.argv.pop(1)
    if arg == '--config':
        # If we pass the arguments "--config some_path/scipion.conf",
        # only the config files in that path will be read.
        scipionLocalConfig = scipionConfig = os.path.abspath(os.path.expanduser(value))

        # Verify existence if not config
        if getMode() != MODE_CONFIG and not exists(scipionConfig):
            # Here we can react differently,instead of exiting, may be continuing warning about
            # the missing config file?
            sys.exit('Config file missing: %s' % scipionConfig)

    else:
        sys.exit('Unknown argument: %s' % arg)

# Protocols.conf and hosts.conf, fallback to the template.
protocols = getConfigPathFromConfigFile(scipionConfig, PROTOCOLS)
if not exists(protocols):
    protocols = join(getTemplatesPath(), "protocols.template")

hosts = getConfigPathFromConfigFile(scipionConfig, HOSTS)
if not exists(hosts):
    hosts = join(getTemplatesPath(), "hosts.template")


# *********************** STORE VARIABLES ********************
class Vars:
    """ Class to hold all the variables that are initialized here"""
    SCIPION_DOMAIN = "pwem"

    # Installation paths
    SCIPION_HOME = scipionHome

    # Scipion path to its own scripts
    SCIPION_SCRIPTS = getScriptsPath()
    # Scipion path to install
    SCIPION_INSTALL = getInstallPath()

    # Config files
    SCIPION_CONFIG = scipionConfig
    SCIPION_LOCAL_CONFIG = scipionLocalConfig
    SCIPION_PROTOCOLS = protocols
    SCIPION_HOSTS = os.environ.get('SCIPION_HOSTS', hosts)

    # Paths to apps or scripts
    PW_APPS = join(getModuleFolder("pyworkflow"), 'apps')
    SCIPION_TEMPLATES = getTemplatesPath()

    SCIPION_VERSION = getVersion()
    SCIPION_PYTHON = PYTHON
    SCIPION_TESTS_CMD = os.environ.get("SCIPION_TESTS_CMD", '%s %s' % (SCIPION_EP, MODE_TESTS))


# *********************** READ CONFIG FILES ***********************
try:
    VARS = dict()

    # Load variables from Vars class into VARS dict

    if 'SCIPION_NOGUI' in os.environ:
        # This cannot work since pyworkflow is not imported and can not be imported here
        # Due to a wrong/early initialisation of the config
        #PYTHONPATH_LIST.insert(0, join(pyworkflow.Config.getPyworkflowPath(), 'gui', 'no-tkinter'))
        print("SCIPION_NOGUI variable not implemented for this version. Please contact us if you need this.")


    # Load VARS dictionary, all items here will go to the environment
    VARS['SCIPION_DOMAIN'] = Vars.SCIPION_DOMAIN
    VARS['SCIPION_CONFIG'] = Vars.SCIPION_CONFIG
    VARS['SCIPION_LOCAL_CONFIG'] = Vars.SCIPION_LOCAL_CONFIG
    VARS['SCIPION_PROTOCOLS'] = Vars.SCIPION_PROTOCOLS
    VARS['SCIPION_HOSTS'] = Vars.SCIPION_HOSTS
    VARS['SCIPION_VERSION'] = Vars.SCIPION_VERSION

    # Read main config file
    config2Dict(Vars.SCIPION_CONFIG, VARS)

    # Load the local config
    if Vars.SCIPION_LOCAL_CONFIG != Vars.SCIPION_CONFIG:
        config2Dict(Vars.SCIPION_LOCAL_CONFIG, VARS)

except Exception as e:
    if len(sys.argv) == 1 or sys.argv[1] != MODE_CONFIG:
        print('Error reading config: %s\n' % e)
        print('Please check the configuration file %s and '
                         'try again.\n' % Vars.SCIPION_CONFIG)
        sys.exit(1)

def main():
    printVersion()
    # See in which "mode" the script is called. By default, it's MODE_MANAGER.
    n = len(sys.argv)
    # Default to MANAGER_MODE
    mode = getMode()

    # Prepare the environment
    os.environ.update(VARS)

    # Trigger Config initialization once environment is ready
    import pyworkflow
    pwVARS = pyworkflow.Config.getVars()
    VARS.update(pwVARS)

    # Update the environment now with pyworkflow values.
    os.environ.update(VARS)

    # Check mode
    if mode == MODE_MANAGER:
        from pyworkflow.gui.project import ProjectManagerWindow
        from scipion.install.update_manager import UpdateManager
        UpdateManager.getPackagesStatus(printAll=False)
        ProjectManagerWindow().show()

    elif mode == MODE_LAST:
        os.environ.update(VARS)
        from pyworkflow.apps.pw_project import openProject
        openProject('last')

    elif mode == MODE_HERE:
        os.environ.update(VARS)
        from pyworkflow.apps.pw_project import openProject
        openProject('here')
        
    elif mode == MODE_PROJECT:
        os.environ.update(VARS)
        from pyworkflow.apps.pw_project import openProject
        openProject(sys.argv[2])

    elif mode == MODE_TESTS or mode == MODE_TEST:
        os.environ.update(VARS)
        from pyworkflow.apps.pw_run_tests import Tester
        Tester().main(sys.argv[2:])

    elif mode == MODE_TEST_DATA:
        runApp('pw_sync_data.py', args=sys.argv[2:])

    elif mode in MODE_VIEWER:
        runApp('pw_viewer.py', args=sys.argv[2:], chdir=False)

    # elif mode == MODE_INSTALL_BINS:
    #     runScript('scipion install %s' % ' '.join(sys.argv[2:]))

    elif mode in PLUGIN_MODES:
        os.environ.update(VARS)
        from scipion.install import installPluginMethods
        installPluginMethods()

    elif mode == MODE_PLUGINS:
        runScript(join(Vars.SCIPION_INSTALL, PLUGIN_MANAGER_PY))

    elif mode == MODE_CONFIG:
        from .scripts.config import main
        os.environ.update(VARS)
        main(sys.argv[2:])

    elif mode == MODE_VERSION:
        # Just exit, Scipion version will be printed anyway
        sys.exit(0)

    elif mode == MODE_RUNPROTOCOL:
        assert (n == 6 or n == 7), 'runprotocol takes exactly 5 arguments, not %d' % (n - 1)
        # this could be pw_protocol_run.py or pw_protocol_mpirun.py
        protocolApp = sys.argv[2]
        # This should be (projectPath, protocolDb and protocolId)
        runApp(protocolApp, args=sys.argv[3:])
        
    elif mode == MODE_PROTOCOLS:
        runApp('pw_protocol_list.py', args=sys.argv[2:])

    elif mode == MODE_ENV:
        # Print all the environment variables needed to run scipion.
        from pyworkflow.plugin import Plugin

        # Trigger plugin's variable definition
        pyworkflow.Config.getDomain().getPlugins()
        VARS.update(pyworkflow.plugin.Plugin.getVars())
        for key in sorted(VARS):
            sys.stdout.write('export %s="%s"\n' % (key, VARS[key]))

        sys.exit(0)

    elif mode == MODE_RUN:
        # Run any command with the environment of scipion loaded.
        runCmd('emprogram ' + ' '.join(['"%s"' % arg for arg in sys.argv[2:]]))
        
    elif mode == MODE_PYTHON:
        runScript(' '.join(['"%s"' % arg for arg in sys.argv[2:]]), 
                  chdir=False)

    elif mode == MODE_TUTORIAL:
        runApp(join(Vars.SCIPION_SCRIPTS, 'tutorial.py'), sys.argv[2:])

    elif mode in MODE_DEMO:
        from scipion.scripts.kickoff import main as launchKickoff
        # Remove one arg
        sys.argv = sys.argv[1:]
        launchKickoff()

    # Allow to run programs from different packages
    # scipion will load the specified environment
    elif (mode.startswith('xmipp') or
          mode.startswith('relion') or
          mode.startswith('e2') or 
          mode.startswith('sx') or
          mode.startswith('b')):
        # To avoid Ghost activation warning
        from pwem import EM_PROGRAM_ENTRY_POINT
        runCmd(EM_PROGRAM_ENTRY_POINT,  sys.argv[1:])

    elif mode == MODE_INSPECT:
        runScript(join(Vars.SCIPION_INSTALL, 'inspect-plugins.py'), sys.argv[2:])

    elif mode == MODE_UPDATE:
        # Once more: local import to avoid importing pyworkflow, triegered by install.__init__ (Plugin Manager)
        from scipion.install.update_manager import updateManagerParser
        updateManagerParser(sys.argv[:])
    # Else HELP or wrong argument
    else:
        sys.stdout.write("""\
Usage: scipion [--config PATH] [MODE] [ARGUMENTS]

    --config               Path to a full config file
                    
MODE can be:
    help                   Prints this help message.

    config                 Checks and/or writes Scipion's global and local configuration.
    
    plugins                Launches the plugin manager window.
    
    installp               Installs Scipion plugins from a terminal. Use flag --help to see usage.
    
    uninstallp             Uninstalls Scipion plugins from a terminal. Use with flag --help to see usage.
    
    installb               Installs Plugin Binaries. Use with flag --help to see usage.

    manager                Opens the manager with a list of all projects.

    inspect                inspect a python module and check if it looks loke a scipion plugin. 
    
    printenv               Prints the environment variables used by the application.

    project NAME           Opens the specified project. The name 'last' opens the last project.

    last                   Same as 'project last'

    run COMMAND [ARG ...]  Runs COMMAND within the Scipion environment.
    
    python [ARG ...]       Shortcut for 'scipion run python ...'

    test OPTION            Runs/Lists test(s).
                           OPTION can be:
                             <name>: name of the test to run
                             --show: list the available tests
                             --help: show all the available options
                             --grep <pattern> : filter the list using the <pattern> 
                             --run: run the list off tests. Affected by --grep
                           For example, to run the "test_object" test:
                             scipion test tests.model.test_object

    testdata OPTION        Gets(puts) tests data, from(to) the server to(from) the $SCIPION_TESTS folder.
                           OPTIONS can be:
                             --download: copy dataset from remote location to local
                             --upload: copy dataset from local to remote
                             <dataset>: name of dataset to download, upload or format
                             --list: list the datasets in the local computer and in the server
                             --help: show all the available options
                           For example, to download the dataset xmipp_tutorial:
                             scipion testdata --download xmipp_tutorial
                           Or to upload it:
                             scipion testdata --upload xmipp_tutorial
                             
    tutorial [NAME]        Creates a new protocol with a tutorial workflow loaded.
                           If NAME is empty, the list of available tutorials are shown.

    view | show NAME       Opens a file with Scipion's showj, or a directory with Browser.
    
    %s [TEMPLATE]    Shows all the *.json.template files found in the config folder
                           and all templates provided by plugins. If TEMPLATE 
                           (a path to a template or a template name) is provided, 
                           then that template is used.

    %s [ARGS]          Check for updates of scipion-em, scipion-pyworkflow 
                           and scipion-app and updates them. OPTIONS can be:
                              -h or --help: to see usage.
                              -dry : only check the status of scipion-em, scipion-pyworkflow 
                                     and scipion-app

""" % (MODE_DEMO[1], MODE_UPDATE))

           


        if mode == MODE_HELP:
            sys.exit(0)
        else:
            print("Unknown mode: %s." % mode)
            sys.exit(1)


if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        import traceback
        traceback.print_exc()
        sys.exit('Error at main: %s\n' % e)



#!/usr/bin/env python
# **************************************************************************
# *
# * Authors:    J. Jimenez de la Morena (jjimenez@cnb.csic.es)
# *             Yunior C. Fonseca Reyna (cfonseca@cnb.csic.es)
# *
# *  [1] Unidad de Bioinformatica of Centro Nacional de Biotecnologia, CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
"""
This module is responsible for updating scipion-em, scipion-pyworkflow and
scipion-app if a higher version of these is released
"""
import argparse
from threading import Thread

from pip._internal.commands import create_command

from pyworkflow.utils import redStr, greenStr, os
from scipion.constants import MODE_UPDATE

DRY_COMMAND = '-dry'
SCIPION_NAME = 'Scipion'


def updateManagerParser(args):
    """
     Create the parser for the "update" command
    """
    parser = argparse.ArgumentParser(prog=args[1:],
                                     formatter_class=argparse.RawTextHelpFormatter)
    subparsers = parser.add_subparsers()
    parser_f = subparsers.add_parser(MODE_UPDATE,
                                     description='description: update {}.'.format(
                                         SCIPION_NAME),
                                     formatter_class=argparse.RawTextHelpFormatter,
                                     usage="{} [-h/--help] [{}]".format(
                                         ' '.join(args[:2]), DRY_COMMAND)
                                     )
    parser_f.add_argument(DRY_COMMAND,
                          help='only check status {}.'.format(SCIPION_NAME),
                          action="store_true")

    parsedArgs = parser.parse_args(args[1:])
    outdatedPackages = UpdateManager.getPackagesStatus()
    if not outdatedPackages:
        print('{} is up to date.'.format(SCIPION_NAME))
    elif not parsedArgs.dry:
        UpdateManager.updateScipion(outdatedPackages)


class UpdateManager:
    """
    Class responsible for updating scipion-em, scipion-pyworkflow and
    scipion-app if a higher version of these is released
    """
    import pyworkflow
    import pwem
    import scipion

    packageNames = [('scipion-pyworkflow', pyworkflow.__version__),
                    ('scipion-em', pwem.__version__),
                    ('scipion-app', scipion.__version__)]

    @classmethod
    def getPackagesStatus(cls, printAll=True):
        """
        Check for scipion-app, scipion-pyworkflow and scipion-em updates
        return: a list of modules to be updated
        """
        def check():
            outdatedPackages = []
            for package in cls.packageNames:
                
                needToUpdate, version = cls.getPackageState(package[0],
                                                                package[1])
                if needToUpdate:
                    outdatedPackages.append((package[0], version))
                    print(
                        redStr('The package %s is out of date. Your version is %s, '
                               'the latest is %s.' % (package[0], package[1],
                                                      version)))
                elif printAll:
                    print(greenStr('The package %s is up to date.  Your version '
                                   'is %s' % (package[0], version)))
    
            return outdatedPackages

        thread = Thread(target=check)
        thread.start()

    @classmethod
    def getPackageState(cls, packageName, version):
        """
        Check if a package needs to be updated or not
        args: packageName: the package name
              version: version of the installed package
        return: (True, version) if the the package needs to be updated, otherwise
                (False, version)

        """
        # Ignore autocheck of outeaded package that happens at import time
        os.environ["OUTDATED_IGNORE"] = "1"
        from outdated import check_outdated
        from requests.exceptions import ConnectionError
        try:
            checkOutdated = check_outdated(packageName, version)
        except ConnectionError as connError:
            print("Cannot check update status of %s (%s)" % (packageName, version))
            return False, version
        except ValueError:
            # We intentionally skip this error
            # When working in devel mode with an increased version not yet release this Value error
            # happens: --> example: Version 3.0.2 is greater than the latest version on PyPI: 3.0.1
            return  False, version
        except Exception as ex:
            print(redStr('%s :%s' % (packageName, ex)))
            return False, version

        return checkOutdated

    @classmethod
    def updateScipion(cls, outdatedPackages):
        """
        Update a module from which there is released a higher version
        """
        kwargs = {'isolated': False}

        for packageName in outdatedPackages:
            cmd_args = [packageName[0],
                        '--upgrade',
                        '-vvv']  # highest level of verbosity

            command = create_command('install', **kwargs)
            status = command.main(cmd_args)
            if status == 0:
                print('%s was correctly updated.' % packageName[0])
            else:
                print('Something went wrong during the update of %s.'
                      % packageName[0])



# NOTE: This is far from ideal.
# We want to register actions in pyworkflow windows.
# Since pyworkflow scans packages, this init will be triggered by pyworkflow and then
# we will register the menu (only works for the project window and not for the "project list" window.
# register plugin menus
import os
from scipion.utils import getInstallPath, getScriptsPath
from scipion.constants import PLUGIN_MANAGER_PY, PYTHON, KICKOFF

def launchPluginManager():
    os.system("%s %s" % (PYTHON, os.path.join(getInstallPath(), PLUGIN_MANAGER_PY)))

def launchTemplates():
    os.system("%s %s" % (PYTHON, os.path.join(getScriptsPath(), KICKOFF)))

# Cancel this for now to prevent an early import of Config and a false initialization
from pyworkflow.gui.project import ProjectManagerWindow
ProjectManagerWindow.registerPluginMenu("Plugin manager", launchPluginManager, None)
ProjectManagerWindow.registerPluginMenu("Workflow templates", launchTemplates, None)



# -*- coding: utf-8 -*
# **************************************************************************
# *
# * Authors: Jorge Jimńenez de la Morena    (jjimenez@cnb.csic.es)
# *
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
import sys
from os.path import join, dirname, exists, isdir
from os import environ
import importlib


def getScipionHome():
    home = environ.get("SCIPION_HOME", None)

    if not home:
        sys.exit("SCIPION_HOME environment variable must be set")

    if not exists(home):
        sys.exit("SCIPION_HOME value (%s) does not exists." % home)

    if not isdir(home):
        sys.exit("SCIPION_HOME value (%s) is not a folder." % home)

    return home


def getScipionAppPath():
    return dirname(__file__)


def getInstallPath():
    return join(getScipionAppPath(), 'install')


def getScriptsPath():
    return join(getScipionAppPath(), 'scripts')


def getTemplatesPath():
    return join(getScipionAppPath(), 'templates')


def getExternalJsonTemplates():
    import pyworkflow
    return dirname(pyworkflow.Config.SCIPION_CONFIG)


def getModuleFolder(moduleName):
    """ Returns the path of a module without importing it"""
    spec = importlib.util.find_spec(moduleName)
    return dirname(spec.origin)



# **************************************************************************
# *
# * Authors: J. Burguet Castell (jburguet@cnb.csic.es)
# *
# * Unidad de Bioinformatica of Centro Nacional de Biotecnologia, CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# **************************************************************************
"""
Check the local configuration files, and/or create them if requested
or if they do not exist.
"""
import sys
import os
from os.path import join, exists, basename
import time
import optparse
from pathlib import Path

from configparser import ConfigParser  # Python 3
from shutil import copyfile

from scipion.utils import getTemplatesPath


PYWORKFLOW_SECTION = "PYWORKFLOW"
SCIPION_CONF = 'scipion'
BACKUPS = 'backups'
HOSTS = 'hosts'
PROTOCOLS = 'protocols'
MISSING_VAR = "None"
SCIPION_NOTIFY = 'SCIPION_NOTIFY'
SCIPION_CONFIG = 'SCIPION_CONFIG'
SCIPION_LOCAL_CONFIG = 'SCIPION_LOCAL_CONFIG'


UPDATE_PARAM = '--update'
COMPARE_PARAM = '--compare'


def ansi(n):
    """Return function that escapes text with ANSI color n."""
    return lambda txt: '\x1b[%dm%s\x1b[0m' % (n, txt)


black, red, green, yellow, blue, magenta, cyan, white = map(ansi, range(30, 38))


# We don't take them from pyworkflow.utils because this has to run
# with all python versions (and so it is simplified).


def main(args=None):
    parser = optparse.OptionParser(description=__doc__)
    add = parser.add_option  # shortcut
    add('--overwrite', action='store_true',
        help="Rewrite the configuration files using the original templates.")
    add(UPDATE_PARAM, action='store_true',
        help=("Updates you local config files with the values in the template, "
              "only for those missing values."))
    add('--notify', action='store_true',
        help="Allow Scipion to notify usage data (skips user question) "
             "TO BE DEPRECATED, use unattended param instead")
    add('--unattended', action='store_true',
        help="Scipion will skipping questions")
    add('-p', help='Prints the config variables associated to plugin P')

    add(COMPARE_PARAM, action='store_true',
        help="Check that the configurations seems reasonably well set up.")

    options, args = parser.parse_args(args)

    if args:  # no args which aren't options
        sys.exit(parser.format_help())

    unattended = options.notify or options.unattended

    if options.p:
        import pyworkflow.utils as pwutils
        pluginName = options.p
        plugin = (pwutils.Config.getDomain().
                  importFromPlugin(pluginName, 'Plugin'))

        if plugin is not None:
            plugin._defineVariables()

            print("Variables defined by plugin '%s':\n" % pluginName)
            for k, v in plugin._vars.items():
                print("%s = %s" % (k, v))
            print("\nThese variables can be added/edited in '%s'"
                  % os.environ[SCIPION_CONFIG])
        else:
            print("No plugin found with name '%s'. Module name is expected,\n"
                  "i.e. 'scipion3 config -p xmipp3' shows the config variables "
                  "defined in 'scipion-em-xmipp' plugin." % pluginName)

        sys.exit(0)

    try:
        # where templates are
        templates_dir = getTemplatesPath()

        scipionConfigFile = os.environ[SCIPION_CONFIG]
        # Global installation configuration files.
        for fpath, tmplt in [
            (scipionConfigFile, SCIPION_CONF),
            (getConfigPathFromConfigFile(scipionConfigFile,PROTOCOLS),  PROTOCOLS),
            (getConfigPathFromConfigFile(scipionConfigFile,HOSTS), HOSTS)]:
            if not exists(fpath) or options.overwrite:
                print(fpath, tmplt)
                createConf(fpath, join(templates_dir, getTemplateName(tmplt)),
                           unattended=unattended)
            else:
                checkConf(fpath, join(templates_dir, getTemplateName(tmplt)),
                          update=options.update,
                          unattended=unattended)

        # Check paths for the config
        checkPaths(os.environ[SCIPION_CONFIG])

    except Exception as e:
        # This way of catching exceptions works with Python 2 & 3
        print('Config error: %s\n' % e)
        import traceback
        traceback.print_exc()
        sys.exit(1)


def getTemplateName(template):
    return template + '.template'


def checkNotify(config, configfile, unattended):
    """ Check if protocol statistics should be collected. """

    print("""--------------------------------------------------------------
-----------------------------------------------------------------
It would be very helpful if you allow Scipion to send anonymous usage data. This
information will help Scipion's team to identify the more demanded protocols and
prioritize support for them.

Collected usage information is COMPLETELY ANONYMOUS and does NOT include protocol
parameters, files or any data that can be used to identify you or your data. At
https://scipion-em.github.io/docs/docs/developer/collecting-statistics.html you
may see examples of the transmitted data as well as the statistics created with it.
You can always deactivate/activate this option by editing the file %s and setting 
the variable SCIPION_NOTIFY to False/True respectively.

We understand, of course, that you may not wish to have any information collected
from you and we respect your privacy.
""" % configfile)

    if not unattended:
        input("Press <enter> to continue:")

    config.set(PYWORKFLOW_SECTION, SCIPION_NOTIFY, 'True')


def createConf(fpath, ftemplate, unattended=False):
    """Create config file in fpath following the template in ftemplate"""
    # Remove from the template the sections in "remove", and if "keep"
    # is used only keep those sections.

    # Create directory and backup if necessary.
    dname = os.path.dirname(fpath)
    if not exists(dname):
        os.makedirs(dname)
    elif exists(fpath):
        if not exists(join(dname, BACKUPS)):
            os.makedirs(join(dname, BACKUPS))
        backup = join(dname, BACKUPS,
                      '%s.%d' % (basename(fpath), int(time.time())))
        print(yellow("* Creating backup: %s" % backup))
        os.rename(fpath, backup)

    # Read the template configuration file.
    print(yellow("* Creating configuration file: %s" % fpath))
    print("Please edit it to reflect the configuration of your system.\n")

    if not os.path.exists(ftemplate):
        raise FileNotFoundError('Missing file: %s' % ftemplate)

    # Special case for scipion config
    if getTemplateName(SCIPION_CONF) in ftemplate:

        cf = ConfigParser()
        cf.optionxform = str  # keep case (stackoverflow.com/questions/1611799)

        addPyworkflowVariables(cf)
        addPluginsVariables(cf)

        # Collecting protocol Usage Statistics
        checkNotify(cf, fpath, unattended=unattended)

        # Create the actual configuration file.
        cf.write(open(fpath, 'w'))
    else:
        # For host.conf and protocols.conf, just copy files
        copyfile(ftemplate, fpath)

def addVariablesToSection(cf, section, vars, exclude=[]):
    """ Add all the variables in vars to the config "cf" at the section passed
    it cleans the path to avoid long absolute repetitive paths"""

    def cleanVarPath(varValue):
        """ Clean variable to avoid long paths and relate them to SCIPION_HOME or EM_ROOT"""
        import pwem
        import pyworkflow as pw

        # If it's EM_ROOT, just replace SCIPION_HOME to make it relative
        if varValue == pwem.Config.EM_ROOT:
            varValue = varValue.replace(pwem.Config.SCIPION_HOME, "")
            if varValue.startswith(os.path.sep):
                varValue = varValue [1:]

        elif varValue.startswith(pwem.Config.EM_ROOT):
            varValue = varValue.replace(pwem.Config.EM_ROOT, "%(EM_ROOT)s")

        # If value contains SCIPION_HOME and is not scipion home
        if varValue.startswith(pw.Config.SCIPION_HOME) and varValue != pw.Config.SCIPION_HOME:
            varValue = varValue.replace(pwem.Config.SCIPION_HOME, "${SCIPION_HOME}")

        # Replace HOME paths with ~
        home = str(Path.home())
        if varValue.startswith(home):
            varValue = varValue.replace(home, "~")

        return varValue

    cf.add_section(section)
    for var in sorted(vars.keys()):
        if var not in exclude:
            value = vars[var]
            cf.set(section, var, cleanVarPath(str(value)))


def addPyworkflowVariables(cf):
    # Once more we need a local import to prevent the Config to be wrongly initialized
    import pyworkflow as pw

    exclude = ["SCIPION_CONFIG","SCIPION_CWD", "SCIPION_LOCAL_CONFIG",
               "SCIPION_HOME", "SCIPION_PROTOCOLS", "SCIPION_HOSTS"]
    # Load pyworkflow variables from the config
    addVariablesToSection(cf, PYWORKFLOW_SECTION, pw.Config.getVars(), exclude )


def addPluginsVariables(cf):
    # Once more we need a local import to prevent the Config to be wrongly initialized
    import pyworkflow as pw
    from pyworkflow.plugin import Plugin

    # Trigger plugin discovery and variable definition
    pw.Config.getDomain().getPlugins()
    addVariablesToSection(cf, "PLUGINS", Plugin.getVars())


def checkPaths(conf):
    """Check that some paths in the config file actually make sense"""

    print("Checking paths in %s ..." % conf)
    cf = ConfigParser()
    cf.optionxform = str  # keep case (stackoverflow.com/questions/1611799)
    assert cf.read(conf) != [], 'Missing file: %s' % conf

    def get(var):
        try:
            return cf.get('BUILD', var)
        except Exception:
            # Not mandatory anymore
            return MISSING_VAR

    allOk = True

    for fname in [join(get('JAVA_BINDIR'), 'java'),
                  get('JAVAC'), get('JAR'),
                  join(get('MPI_BINDIR'), get('MPI_CC')),
                  join(get('MPI_BINDIR'), get('MPI_CXX')),
                  join(get('MPI_BINDIR'), get('MPI_LINKERFORPROGRAMS')),
                  join(get('MPI_INCLUDE'), 'mpi.h')]:
        if not fname.startswith(MISSING_VAR) and not exists(fname):
            print("  Cannot find file: %s" % red(fname))
            allOk = False
    if allOk:
        print(green("All seems fine with %s" % conf))
    else:
        print(red("Errors found."))
        print("Please edit %s and check again." % conf)
        print("To regenerate the config files trying to guess the paths, you "
              "can run: scipion config --overwrite")


def checkConf(fpath, ftemplate, update=False, unattended=False, compare=False):
    """Check that all the variables in the template are in the config file too
    :parameter fpath:  path to the config file
    :parameter ftemplate, template file to compare. Only used for protocols and hosts
    :parameter update: flag, default to false. if true, values from the template will be written to the config file
    :parameter unattended: avoid questions, default to false.
    :parameter compare: make a comparison with the template"""
    # Remove from the checks the sections in "remove", and if "keep"
    # is used only check those sections.

    # Read the config file fpath and the template ftemplate
    cf = ConfigParser()
    cf.optionxform = str  # keep case (stackoverflow.com/questions/1611799)
    assert cf.read(fpath) != [], 'Missing file %s' % fpath

    ct = ConfigParser()
    ct.optionxform = str

    suggestUpdate = True # Flag to suggest --update

    # Special case for scipion config... get values from objects
    if getTemplateName(SCIPION_CONF) in ftemplate:
        # This will be the place to "exclude some variables"
        addPyworkflowVariables(ct)
        addPluginsVariables(ct)
    else:
        # Cancel update for others than SCIPION_CONF
        update = False
        suggestUpdate = False
        assert ct.read(ftemplate) != [], 'Missing file %s' % ftemplate

    df = dict([(s, set(cf.options(s))) for s in cf.sections()])
    dt = dict([(s, set(ct.options(s))) for s in ct.sections()])
    # That funny syntax to create the dictionaries works with old pythons.

    if compare:
        compareConfig(cf, ct, fpath, ftemplate)
        return

    confChanged = False

    if df == dt:
        print(green("All the expected sections and options found in " + fpath))
    else:
        print("Found differences between the configuration file\n  %s\n"
              "and the template file\n  %s" % (fpath, ftemplate))
        sf = set(df.keys())
        st = set(dt.keys())
        for s in sf - st:
            print("Section %s exists in the configuration file but "
                  "not in the template." % red(s))

        for s in st - sf:
            print("Section %s exists in the template but not in the configuration file. Use %s parameter to update  "
                  "local config files." % (yellow(s), UPDATE_PARAM))

            if update:
                # Update config file with missing section
                cf.add_section(s)
                # add it to the keys
                sf.add(s)
                df[s] = set()
                print("Section %s added to your config file."
                      % green(s))
                confChanged = True

        for s in st & sf:
            for o in df[s] - dt[s]:
                print("In section %s, option %s exists in the configuration "
                      "file but not in the template." % (red(s), red(o)))
            for o in dt[s] - df[s]:
                suggestion = "" if not suggestUpdate else  " Use %s parameter to update local config files." % UPDATE_PARAM
                print("In section %s, option %s exists in the template but not in the configuration file.%s" % (yellow(s), yellow(o), suggestion))

                if update:
                    if o == 'SCIPION_NOTIFY':
                        checkNotify(ct, fpath, unattended)
                    # Update config file with missing variable
                    value = ct.get(s, o)
                    cf.set(s, o, value)
                    confChanged = True
                    print("Variable %s -> %s added and set to %s in your config file."
                          % (s, green(o), value))

    if update:
        if confChanged:
            print("Changes detected: writing changes into %s. Please check values." % fpath)
        else:
            print("Update requested no changes detected for %s." % fpath)

        try:
            with open(fpath, 'w') as f:
                cf.write(f)
        except Exception as e:
            print("Could not update the config: ", e)


def compareConfig(cf, ct, fPath, fTemplate):
    """ Compare configuration against template values"""

    print(magenta("COMPARING %s to %s" % (fPath, fTemplate)))
    print(magenta("We expect values to follow the <package>-<version> pattern."
                  " If you see any value not following this pattern please "
                  "update it."))
    # loop through the config
    for s in cf.sections():
        # Get the section
        for variable in cf._sections[s]:
            # Get values
            valueInConfig = getConfigVariable(cf, s, variable)
            valueInTemplate = getConfigVariable(ct, s, variable)

            # Compare value with template
            compareConfigVariable(s, variable, valueInConfig, valueInTemplate)


def getConfigVariable(config, section, variableName):
    return config._sections[section].get(variableName)


def compareConfigVariable(section, variableName, valueInConfig, valueInTemplate):
    if valueInTemplate is None:
        return

    if valueInConfig != valueInTemplate:
        print("%s at %s section (%s) differs from the default value in the "
              "template: %s" % (red(variableName), section, red(valueInConfig),
                                yellow(valueInTemplate)))


def guessJava():
    """Guess the system's Java installation, return a dict with the Java keys"""

    options = {}
    candidates = []

    # First check if the system has a favorite one.
    if 'JAVA_HOME' in os.environ:
        candidates.append(os.environ['JAVA_HOME'])

    # Add also all the ones related to a "javac" program.
    for d in os.environ.get('PATH', '').split(':'):
        if not os.path.isdir(d) or 'javac' not in os.listdir(d):
            continue
        javaBin = os.path.realpath(join(d, 'javac'))
        if javaBin.endswith('/bin/javac'):
            javaHome = javaBin[:-len('/bin/javac')]
            candidates.append(javaHome)
            if javaHome.endswith('/jre'):
                candidates.append(javaHome[:-len('/jre')])

    # Check in order if for any of our candidates, all related
    # directories and files exist. If they do, that'd be our best guess.
    for javaHome in candidates:
        allExist = True
        for path in ['include', join('bin', 'javac'), join('bin', 'jar')]:
            if not exists(join(javaHome, path)):
                allExist = False
        if allExist:
            options['JAVA_HOME'] = javaHome
            break
            # We could instead check individually for JAVA_BINDIR, JAVAC
            # and so on, as we do with MPI options, but we go for an
            # easier and consistent case instead: everything must be under
            # JAVA_HOME, which is the most common case for Java.

    if not options:
        print(red("Warning: could not detect a suitable JAVA_HOME."))
        if candidates:
            print(red("Our candidates were:\n  %s" % '\n  '.join(candidates)))

    return options


def guessMPI():
    """Guess the system's MPI installation, return a dict with MPI keys"""
    # Returns MPI_LIBDIR, MPI_INCLUDE and MPI_BINDIR as a dictionary.

    options = {}
    candidates = []

    # First check if the system has a favorite one.
    for prefix in ['MPI_', 'MPI', 'OPENMPI_', 'OPENMPI']:
        if '%sHOME' % prefix in os.environ:
            candidates.append(os.environ['%sHOME' % prefix])

    # Add also all the ones related to a "mpicc" program.
    for d in os.environ.get('PATH', '').split(':'):
        if not os.path.isdir(d) or 'mpicc' not in os.listdir(d):
            continue
        mpiBin = os.path.realpath(join(d, 'mpicc'))
        if 'MPI_BINDIR' not in options:
            options['MPI_BINDIR'] = os.path.dirname(mpiBin)
        if mpiBin.endswith('/bin/mpicc'):
            mpiHome = mpiBin[:-len('/bin/mpicc')]
            candidates.append(mpiHome)

    # Add some extra directories that are commonly around.
    candidates += ['/usr/lib/openmpi', '/usr/lib64/mpi/gcc/openmpi']

    # Check in order if for any of our candidates, all related
    # directories and files exist. If they do, that'd be our best guess.
    for mpiHome in candidates:
        if (exists(join(mpiHome, 'include', 'mpi.h')) and
                'MPI_INCLUDE' not in options):
            options['MPI_INCLUDE'] = join(mpiHome, 'include')
        if (exists(join(mpiHome, 'lib', 'libmpi.so')) and
                'MPI_LIBDIR' not in options):
            options['MPI_LIBDIR'] = join(mpiHome, 'lib')
        if (exists(join(mpiHome, 'bin', 'mpicc')) and
                'MPI_BINDIR' not in options):
            options['MPI_BINDIR'] = join(mpiHome, 'bin')

    return options


def getConfigPathFromConfigFile(scipionConfigFile, configFile):
    """
    :param scipionConfigFile path to the config file to derive the folder name from
    :param configFile: name of the template: protocols or hosts so far
    :return theoretical path for the template at the same path as the config file"""
    return  os.path.join(os.path.dirname(scipionConfigFile), configFile + ".conf")

if __name__ == '__main__':
    main()



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path
from scipion import __version__
from scipion.constants import SCIPION_EP

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    name='scipion-app',  # Required
    version=__version__,  # Required
    description='Scipion application. For processing cryo electron microscopy '
                'images and hybrid modelling',  # Required
    long_description=long_description,  # Optional
    url='https://github.com/scipion-em/scipion-app',  # Optional
    author='Scipion team',  # Optional
    author_email='scipion@cnb.csic.es',  # Optional
    keywords='scipion cryoem imageprocessing scipion-3.0',  # Optional
    packages=find_packages(),
    install_requires=[requirements],
    package_data={  # Optional
        'scipion': ['templates/*.template', 'templates/*.json'],
    },
    entry_points={
        'console_scripts': [
            '%s = scipion.__main__:main' % SCIPION_EP,
        ],
        'pyworkflow.guiplugin': 'guiplugin = scipion.guiplugin'
    }
    # package_data={  # Optional
    # #    '': [''],
    # }
)
